/*
Beam-Frame Structural Analysis for Casio 9860SDK version 0.2
Copyright (C) 2011 Toussis Manolis (koppermind at gmail.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "fxlib.h"

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "mylib.h"




double getM(double x,int el,struct Matrix* mF,struct Matrix* mL,int nL)
{
	int w1,type;
	double d1,a,b,c;
	d1=matrixGet(mF,el,18)+matrixGet(mF,el,17)*x;//Mleft+Qleft*x
	for(w1=1;w1<=nL;w1++)
	{
		if((int)matrixGet(mL,w1,1)==el)//if load in element el
		{
			type=(int)matrixGet(mL,w1,2);
			a=matrixGet(mL,w1,3);b=matrixGet(mL,w1,4);
			if(type==1)// P
			{
				if(x>a)
				{
					d1=d1-(x-a)*b;
				};
			}else if(type==2)// uniform
			{
				d1=d1-b*x*x/2.0;
			};
		};
	};
	return d1;
}


double getQ(double x,int el,struct Matrix* mF,struct Matrix* mL,int nL)
{
	int w1,type;
	double d1,a,b,c;
	d1=matrixGet(mF,el,17);//Qleft
	for(w1=1;w1<=nL;w1++)
	{
		if((int)matrixGet(mL,w1,1)==el)//if load in element el
		{
			type=(int)matrixGet(mL,w1,2);
			a=matrixGet(mL,w1,3);b=matrixGet(mL,w1,4);
			if(type==1)// P
			{
				if(x>a)
				{
					d1=d1-b;
				};
			}else if(type==2)// uniform
			{
				d1=d1-b*x;
			};
		};
	};
	return d1;
}


double getN(double x,int el,struct Matrix* mF,struct Matrix* mL,int nL)
{
	int w1,type;
	double d1,a,b,c;
	d1=matrixGet(mF,el,16);//Nleft
	for(w1=1;w1<=nL;w1++)
	{
		if((int)matrixGet(mL,w1,1)==el)//if load in element el
		{
			type=(int)matrixGet(mL,w1,2);
			a=matrixGet(mL,w1,3);b=matrixGet(mL,w1,4);
			if(type==1)// P
			{
				d1=d1;
			}else if(type==2)// uniform
			{
				d1=d1;
			};
		};
	};
	return d1;
}

double getMaterialE(struct StringList* l,int index)
{
    //"Name,ElasticModulus,poisson"
    double d1;
    char buf[32];
    char* bufp;
    d1=0.0;
    ASSERT(index>0 && (index<=(l->count)));
    bufp=slistGet(l,index);
    strList(bufp,',',1,buf);//E in buf
    d1=atof(buf);
    return d1;
}

double getSectionA(struct StringList* l,int index)
{
    //"NameRect,1,Width,Height"
    //"NameCirc,2,Radius"
    //"NameCustom,3,Area,Inertia"
    int w3;
    double d1;
    double a,b;
    char buf[32];
    char* bufp;
    d1=0.0;
    ASSERT(index>0 && (index<=(l->count)));
    bufp=slistGet(l,index);
    strList(bufp,',',1,buf);//type in buf
    w3=atoi(buf);
    if(w3==1)//rectangle
    {
        strList(bufp,',',2,buf);a=atof(buf);//width
        strList(bufp,',',3,buf);b=atof(buf);//height
        d1=a*b;
    }else if(w3==2)//circular
    {
        strList(bufp,',',2,buf);a=atof(buf);//radius
        d1=PI*a*a;
    }else if(w3==3)//custom
    {
        strList(bufp,',',2,buf);a=atof(buf);//area
        d1=a;
    };
    return d1;
}

double getSectionI(struct StringList* l,int index)
{
    //"NameRect,1,Width,Height"
    //"NameCirc,2,Radius"
    //"NameCustom,3,Area,Inertia"
    int w3;
    double d1;
    double a,b;
    char buf[32];
    char* bufp;
    d1=0.0;
    ASSERT(index>0 && (index<=(l->count)));
    bufp=slistGet(l,index);
    strList(bufp,',',1,buf);//type in buf
    w3=atoi(buf);
    if(w3==1)//rectangle
    {
        strList(bufp,',',2,buf);a=atof(buf);//width
        strList(bufp,',',3,buf);b=atof(buf);//height
        d1=a*b*b*b/12.0;
    }else if(w3==2)//circular
    {
        strList(bufp,',',2,buf);a=atof(buf);//radius
        d1=PI*a*a*a*a/4.0;
    }else if(w3==3)//custom
    {
        strList(bufp,',',3,buf);b=atof(buf);//inertia
        d1=b;
    };
    return d1;
}


struct StringList* initSectList(struct StringList* l)
{
    int file;
    FONTCHARACTER PathName[]={'\\','\\','f','l','s','0','\\','S','e','c','t','i','o','n','s','.','d','a','t',0};
    int w1,w2,w3;
    char buf[256];
    file=Bfile_OpenFile(PathName,_OPENMODE_READ);
    if(file<0)//Sections file does not exist.
    {
       l=slistInit(2,32);
       slistSet(l,1,"Rect20x20,1,20.0,20.0");
       slistSet(l,2,"Circ30,2,30.0");
    }else //file exists read data.
    {
        Bfile_ReadFile(file,&w2,sizeof(int),-1);//read number of entries
        l=slistInit(w2,32);
        for(w1=1;w1<=w2;w1++)
        {
            Bfile_ReadFile(file,&w3,sizeof(int),-1);//number of bytes of each entry
            Bfile_ReadFile(file,&buf,w3,-1);//read contents
            buf[w3-1]=0;//add /0 to end
            slistSet(l,w1,buf);
        };
    };
    Bfile_CloseFile(file);
    return l;
}


struct StringList* initMatList(struct StringList* l)
{
    int file;
    FONTCHARACTER PathName[]={'\\','\\','f','l','s','0','\\','M','a','t','e','r','i','a','l','.','d','a','t',0};
    int w1,w2,w3;
    char buf[64];
    file=Bfile_OpenFile(PathName,_OPENMODE_READ);
    if(file<0) //Material file does not exist.
    {
        //each material is : NAME,E(young modulus in kN/m2 or MPa),n poisson ratio
        l=slistInit(12,32);
        slistSet(l,1,"Concrete C12/15,26000.00,0.200");
        slistSet(l,2,"Concrete C16/20,27500.00,0.200");
        slistSet(l,3,"Concrete C20/25,29000.00,0.200");
        slistSet(l,4,"Concrete C25/30,30500.00,0.200");
        slistSet(l,5,"Concrete C30/37,32000.00,0.200");
        slistSet(l,6,"Concrete C35/45,33500.00,0.200");
        slistSet(l,7,"Concrete C40/50,35000.00,0.200");
        slistSet(l,8,"Concrete C45/55,36000.00,0.200");
        slistSet(l,9,"Concrete C50/60,37000.00,0.200");
        slistSet(l,10,"Brass/Bronze,110000.00,0.350");
        slistSet(l,11,"Carbon Fiber,150000.00,0.770");
        slistSet(l,12,"Steel,200000.00,0.300");
    }else //file exists read data
    {
        Bfile_ReadFile(file,&w2,sizeof(int),-1);//read number of entries
        l=slistInit(w2,32);
        for(w1=1;w1<=w2;w1++)
        {
            Bfile_ReadFile(file,&w3,sizeof(int),-1);//number of bytes of each entry
            Bfile_ReadFile(file,&buf,w3,-1);//read contents
            buf[w3-1]=0;//add /0 to end
            slistSet(l,w1,buf);
        }
    }
    Bfile_CloseFile(file);
    return l;
}

struct StringList* saveMatList(struct StringList* l)
{
    int file;
    FONTCHARACTER PathName[]={'\\','\\','f','l','s','0','\\','M','a','t','e','r','i','a','l','.','d','a','t',0};
    int w1,w2,w3,w4;
    char* buf;

    buf=slistToArray(l);
    Bfile_DeleteFile(PathName);
    Bfile_CreateFile(PathName,l->arraySize);
    file=Bfile_OpenFile(PathName,_OPENMODE_WRITE);
    if(file>=0)
    {
      Bfile_WriteFile(file,buf,l->arraySize);
    };
    Bfile_CloseFile(file);
    slistFree(l);
    free(buf);
    l=NULL;
    return l;
}

struct StringList* saveSectList(struct StringList* l)
{
    int file;
    FONTCHARACTER PathName[]={'\\','\\','f','l','s','0','\\','S','e','c','t','i','o','n','s','.','d','a','t',0};
    char* buf;

    buf=slistToArray(l);
    Bfile_DeleteFile(PathName);
    Bfile_CreateFile(PathName,l->arraySize);
    file=Bfile_OpenFile(PathName,_OPENMODE_WRITE);
    if(file>=0)
    {
      Bfile_WriteFile(file,buf,l->arraySize);
    };
    Bfile_CloseFile(file);
    slistFree(l);
    free(buf);
    l=NULL;
    return l;
}

char* getNodeNames(int N)
{
    int w1,s,i;
    char* bufp;
    char buf[16];
    bufp=(char *)malloc(N*4+1);//+1 if N=0
    ASSERT(bufp!=NULL);
    i=0;
    bufp[0]=0;
    for(w1=1;w1<=N;w1++)
    {
        sprintf(buf,"%d",w1);//node number
        bufp=strcat(bufp,buf);
        if(w1!=N) bufp=strcat(bufp,",");
    };
    return bufp;
}

char* getElementNames(int E)
{
	int w1,s,i;
	char* bufp;
	char buf[16];
	bufp=(char *)malloc(E*4+1);
	ASSERT(bufp!=NULL);
	i=0;
	bufp[0]=0;
	for(w1=1;w1<=E;w1++)
	{
		sprintf(buf,"%d",w1);//elem number
		bufp=strcat(bufp,buf);
		if(w1!=E) bufp=strcat(bufp,",");
	};
	return bufp;
}

char* getListNames(struct StringList* l)
{
    int w1,n;
    char* buf1;
    char* buf;
    char buf2[128];
    n=0;
    for(w1=1;w1<=l->count;w1++)
    {
        buf1=slistGet(l,w1);
        strList(buf1,',',0,buf2);
        n=n+strlen(buf2)+1+1;//count len + term zero + comma
    };
    buf=(char *)malloc(sizeof(char)*(n+1));
    ASSERT(buf!=NULL);
    buf[0]=0;
    for(w1=1;w1<=l->count;w1++)
    {
        buf1=slistGet(l,w1);
	strList(buf1,',',0,buf2);
	buf=strcat(buf,buf2);//append list item name to buf
        if(w1!=l->count) buf=strcat(buf,",");
    };
    return buf;
}

struct StringList* sectionEdit(struct StringList* l)
{
    int w1,w2,w3,n;
    char* buf1=NULL;
    char* buf2=NULL;
    char buf3[64];
    char b1[64];
    char b2[64];
    char b3[64];
    char b4[64];
    char* bufp;
    double d1;
    struct StringList* dlg1=NULL;
    struct StringList* dlg2=NULL;
    char* b;

    dlg1=slistInit(2,32);
loop1:
    if(buf1!=NULL) {free(buf1);buf1=NULL;};
    buf1=getListNames(l);//get the list of section names
    n=l->count;
    if(buf2!=NULL) {free(buf2);buf2=NULL;};
    buf2=(char *)malloc(strlen(buf1)+32);
    sprintf(buf2,"LIST,8,8,115,45,%d,%s,1",n,buf1);
    slistSet(dlg1,1,buf2);
    slistSet(dlg1,2,"MENU,Add,Edit,Del,,,Done");
    w1=dialogExec(dlg1,"Section List");
    if(w1==101)//Add
    {
        if(n>=20)
        {
            messageBox("Too many sections. >=20","Error");
            goto loop1;
        };
        w2=multipleChoiceBox("Rectangular,Circular,Custom",1);
        if(w2==1)//rectangular
        {
            dlg2=slistInit(8,32);
            slistSet(dlg2,1,"LABEL,1,8,Section:");
            slistSet(dlg2,2,"EDIT,50,8,70,8,Name,%s");
            slistSet(dlg2,3,"LABEL,1,18,Width (cm):");
            slistSet(dlg2,4,"EDIT,50,18,70,8,0.00,%.2f");
            slistSet(dlg2,5,"LABEL,1,28,Height (cm):");
            slistSet(dlg2,6,"EDIT,50,28,70,8,0.00,%.2f");
            slistSet(dlg2,7,"BUTTON,20,50,40,10,OK");
            slistSet(dlg2,8,"BUTTON,75,50,40,10,Cancel");
            dialogSetValue(dlg2,2,"Name");
            dialogSetValue(dlg2,4,"1.00");
            dialogSetValue(dlg2,6,"1.00");
            loop2:
            w3=dialogExec(dlg2,"Add Rectangular Section");
            if(w3==7)// ok pressed
            {
                dialogGetValue(dlg2,2,b1);//name
                if(strlen(b1)>15) b1[15]=0;//trim name
                if(strlen(b1)==0)
                {
                    messageBox("Please enter a section name.","Error");
                    goto loop2;
                };
                dialogGetValue(dlg2,4,b2);d1=atof(b2);//width
                if(d1<=0.0)
                {
                    messageBox("Width cannot be zero or negative.","Error");
                    goto loop2;
                };
                dialogGetValue(dlg2,6,b3);d1=atof(b3);//height
                if(d1<=0.0)
                {
                    messageBox("Height cannot be zero or negative.","Error");
                    goto loop2;
                };
                sprintf(buf2,"%s,1,%s,%s",b1,b2,b3);//buf2 holds the new section
                slistAppend(l,buf2);
            };
            slistFree(dlg2);dlg2=NULL;
        }else if(w2==2)//circular
        {
            dlg2=slistInit(6,32);
            slistSet(dlg2,1,"LABEL,1,8,Section:");
            slistSet(dlg2,2,"EDIT,50,8,70,8,Name,%s");
            slistSet(dlg2,3,"LABEL,1,18,Radius (cm):");
            slistSet(dlg2,4,"EDIT,50,18,70,8,0.00,%.2f");
            slistSet(dlg2,5,"BUTTON,20,50,40,10,OK");
            slistSet(dlg2,6,"BUTTON,75,50,40,10,Cancel");
            dialogSetValue(dlg2,2,"Name");
            dialogSetValue(dlg2,4,"1.00");
            loop3:
            w3=dialogExec(dlg2,"Add Circular Section");
            if(w3==5)// ok pressed
            {
                dialogGetValue(dlg2,2,b1);//name
                if(strlen(b1)>15) b1[15]=0;//trim name
                if(strlen(b1)==0)
                {
                    messageBox("Please enter a section name.","Error");
                    goto loop3;
                };
                dialogGetValue(dlg2,4,b2);d1=atof(b2);//radius
                if(d1<=0.0)
                {
                    messageBox("Radius cannot be zero or negative.","Error");
                    goto loop3;
                };
                sprintf(buf2,"%s,2,%s",b1,b2);//buf2 holds the new section
                slistAppend(l,buf2);
            };
           slistFree(dlg2);dlg2=NULL;
        }else if(w2==3)//custom
        {
            dlg2=slistInit(8,32);
            slistSet(dlg2,1,"LABEL,1,8,Section:");
            slistSet(dlg2,2,"EDIT,50,8,70,8,Name,%s");
            slistSet(dlg2,3,"LABEL,1,18,Area (cm2):");
            slistSet(dlg2,4,"EDIT,50,18,70,8,0.00,%.2f");
            slistSet(dlg2,5,"LABEL,1,28,Inertia (cm4):");
            slistSet(dlg2,6,"EDIT,50,28,70,8,0.00,%.2f");
            slistSet(dlg2,7,"BUTTON,20,50,40,10,OK");
            slistSet(dlg2,8,"BUTTON,75,50,40,10,Cancel");
            dialogSetValue(dlg2,2,"Name");
            dialogSetValue(dlg2,4,"1.00");
            dialogSetValue(dlg2,6,"1.00");
            loop4:
            w3=dialogExec(dlg2,"Add Custom Section");
            if(w3==7)// ok pressed
            {
                dialogGetValue(dlg2,2,b1);//name
                if(strlen(b1)>15) b1[15]=0;//trim name
                if(strlen(b1)==0)
                {
                    messageBox("Please enter a section name.","Error");
                    goto loop4;
                };
                dialogGetValue(dlg2,4,b2);d1=atof(b2);//area
                if(d1<=0.0)
                {
                    messageBox("Area cannot be zero or negative.","Error");
                    goto loop4;
                };
                dialogGetValue(dlg2,6,b3);d1=atof(b3);//inertia
                if(d1<=0.0)
                {
                    messageBox("Inertia cannot be zero or negative.","Error");
                    goto loop4;
                };
                sprintf(buf2,"%s,3,%s,%s",b1,b2,b3);//buf2 holds the new section
                slistAppend(l,buf2);
            };
            slistFree(dlg2);dlg2=NULL;
        };
        goto loop1;
    }else if(w1==102)//edit
    {
        dialogGetValue(dlg1,1,buf3);w2=atoi(buf3);//current section
        bufp=slistGet(l,w2);//current section
        sprintf(b1,"%s",bufp);
        strList(bufp,',',0,b1);//name
        strList(bufp,',',1,b2);//type
        if(atoi(b2)==1)//rectangular
        {
            strList(bufp,',',2,b3);//width
            strList(bufp,',',3,b4);//height
            dlg2=slistInit(8,32);
            slistSet(dlg2,1,"LABEL,1,8,Section:");
            slistSet(dlg2,2,"EDIT,50,8,70,8,Name,%s");
            slistSet(dlg2,3,"LABEL,1,18,Width (cm):");
            slistSet(dlg2,4,"EDIT,50,18,70,8,0.00,%.2f");
            slistSet(dlg2,5,"LABEL,1,28,Height (cm):");
            slistSet(dlg2,6,"EDIT,50,28,70,8,0.00,%.2f");
            slistSet(dlg2,7,"BUTTON,20,50,40,10,OK");
            slistSet(dlg2,8,"BUTTON,75,50,40,10,Cancel");
            dialogSetValue(dlg2,2,b1);
            dialogSetValue(dlg2,4,b3);
            dialogSetValue(dlg2,6,b4);
            loop5:
            w3=dialogExec(dlg2,"Edit Rectangular Section");
            if(w3==7)//ok pressed
            {
                dialogGetValue(dlg2,2,b1);//name
                if(strlen(b1)>15) b1[15]=0;//trim name
                if(strlen(b1)==0)
                {
                    messageBox("Please enter a section name.","Error");
                    goto loop5;
                };
                dialogGetValue(dlg2,4,b3);d1=atof(b3);//width
                if(d1<=0.0)
                {
                    messageBox("Width cannot be zero or negative.","Error");
                    goto loop5;
                };
                dialogGetValue(dlg2,6,b4);d1=atof(b4);//height
                if(d1<=0.0)
                {
                    messageBox("Height cannot be zero or negative.","Error");
                    goto loop5;
                };
                sprintf(buf2,"%s,1,%s,%s",b1,b3,b4);//buf2 holds the new section
                slistSet(l,w2,buf2);
            };
        }else if(atoi(b2)==2)//circular
        {
            strList(bufp,',',2,b3);//radius
            dlg2=slistInit(6,32);
            slistSet(dlg2,1,"LABEL,1,8,Section:");
            slistSet(dlg2,2,"EDIT,50,8,70,8,Name,%s");
            slistSet(dlg2,3,"LABEL,1,18,Radius (cm):");
            slistSet(dlg2,4,"EDIT,50,18,70,8,0.00,%.2f");
            slistSet(dlg2,5,"BUTTON,20,50,40,10,OK");
            slistSet(dlg2,6,"BUTTON,75,50,40,10,Cancel");
            dialogSetValue(dlg2,2,b1);
            dialogSetValue(dlg2,4,b3);
            loop6:
            w3=dialogExec(dlg2,"Edit Circular Section");
            if(w3==5)
            {
                dialogGetValue(dlg2,2,b1);//name
                if(strlen(b1)>15) b1[15]=0;//trim name
                if(strlen(b1)==0)
                {
                    messageBox("Please enter a section name.","Error");
                    goto loop6;
                };
                dialogGetValue(dlg2,4,b3);d1=atof(b3);//radius
                if(d1<=0.0)
                {
                    messageBox("Radius cannot be zero or negative.","Error");
                    goto loop6;
                };
                sprintf(buf2,"%s,2,%s",b1,b3);//buf2 holds the new section
                slistSet(l,w2,buf2);
            };
        }else if(atoi(b2)==3)//custom
        {
            strList(bufp,',',2,b3);//area
            strList(bufp,',',3,b4);//inertia
            dlg2=slistInit(8,32);
            slistSet(dlg2,1,"LABEL,1,8,Section:");
            slistSet(dlg2,2,"EDIT,50,8,70,8,Name,%s");
            slistSet(dlg2,3,"LABEL,1,18,Area (cm2):");
            slistSet(dlg2,4,"EDIT,50,18,70,8,0.00,%.2f");
            slistSet(dlg2,5,"LABEL,1,28,Inertia (cm4):");
            slistSet(dlg2,6,"EDIT,50,28,70,8,0.00,%.2f");
            slistSet(dlg2,7,"BUTTON,20,50,40,10,OK");
            slistSet(dlg2,8,"BUTTON,75,50,40,10,Cancel");
            dialogSetValue(dlg2,2,b1);
            dialogSetValue(dlg2,4,b3);
            dialogSetValue(dlg2,6,b4);
            loop7:
            w3=dialogExec(dlg2,"Edit Custom Section");
            if(w3==7)//ok pressed
            {
                dialogGetValue(dlg2,2,b1);//name
                if(strlen(b1)>15) b1[15]=0;//trim name
                if(strlen(b1)==0)
                {
                    messageBox("Please enter a section name.","Error");
                    goto loop7;
                };
                dialogGetValue(dlg2,4,b3);d1=atof(b3);//area
                if(d1<=0.0)
                {
                    messageBox("Area cannot be zero or negative.","Error");
                    goto loop7;
                };
                dialogGetValue(dlg2,6,b4);d1=atof(b4);//inertia
                if(d1<=0.0)
                {
                    messageBox("Inertia cannot be zero or negative.","Error");
                    goto loop7;
                };
                sprintf(buf2,"%s,3,%s,%s",b1,b3,b4);//buf2 holds the new section
                slistSet(l,w2,buf2);
            };
        };
        slistFree(dlg2);dlg2=NULL;
        goto loop1;
    }else if(w1==103)//del
    {
        if(n==1)
        {
            messageBox("Cannot delete the last| section.","Error");
            goto loop1;
        };
        dialogGetValue(dlg1,1,buf3);w2=atoi(buf3);//w2 holds the current section
        slistRemoveAt(l,w2);
        goto loop1;
    };
    slistFree(dlg1);dlg1=NULL;
    if(buf1!=NULL) free(buf1);
    if(buf2!=NULL) free(buf2);
    return l;
}

struct StringList* materialEdit(struct StringList* l)
{
    int w1,w2,w3,n;
    char* buf1=NULL;
    char* buf2=NULL;
    char buf3[128];
    char b1[64];
    char b2[64];
    char b3[64];
    char* bufp;
    double d1;
    struct StringList* dlg1=NULL;
    struct StringList* dlg2=NULL;
    char* b;
    dlg2=slistInit(8,32);
    slistSet(dlg2,1,"LABEL,1,8,Material:");
	slistSet(dlg2,2,"EDIT,50,8,70,8,Name,%s");
	slistSet(dlg2,3,"LABEL,1,20,Elast.Modulus (MPa or kN/m2):");
	slistSet(dlg2,4,"EDIT,50,30,70,8,0.00,%.2f");
	slistSet(dlg2,5,"LABEL,1,40,Poisson Ratio:");
	slistSet(dlg2,6,"EDIT,70,40,40,8,0.000,%.3f");
	slistSet(dlg2,7,"BUTTON,20,50,40,10,OK");
	slistSet(dlg2,8,"BUTTON,75,50,40,10,Cancel");
    dlg1=slistInit(2,32);
loop1:
    buf1=getListNames(l);
    n=l->count;
    buf2=(char *)malloc(strlen(buf1)+32);
    ASSERT(buf2!=NULL);
    sprintf(buf2,"LIST,8,8,115,45,%d,%s,1",n,buf1);
    slistSet(dlg1,1,buf2);
    free(buf1);free(buf2);
    slistSet(dlg1,2,"MENU,Add,Edit,Del,,,Done");
    w1=dialogExec(dlg1,"Material List");
    if(w1==101)//Add
    {
        if(n>=20)
        {
            messageBox("Too many materials. >=20","Error");
            goto loop1;        
        };
        dialogSetValue(dlg2,2,"Name");//default values
        dialogSetValue(dlg2,4,"1.00");
        dialogSetValue(dlg2,6,"0.500");
        loop2:
        w2=dialogExec(dlg2,"Add Material");
        if(w2==7)//ok pressed
        {
            dialogGetValue(dlg2,2,b1);//name
            if(strlen(b1)>15) b1[15]=0;//trim name
            if(strlen(b1)==0)
            {
              messageBox("Please enter a material name.","Error");
              goto loop2;
            };
            dialogGetValue(dlg2,4,b2);d1=atof(b2);//elastic modulus
            if(d1<=0.0)
            {
              messageBox("Elastic Modulus cannot| be zero or negative.","Error");
              goto loop2;
            };
            dialogGetValue(dlg2,6,b3);
            sprintf(buf3,"%s,%s,%s",b1,b2,b3);//buf2 holds the new material
            slistAppend(l,buf3);
        };
        goto loop1;
    }else if(w1==102)//edit
    {
        dialogGetValue(dlg1,1,buf3);w2=atoi(buf3);//current material
	bufp=slistGet(l,w2);//current material
        strList(bufp,',',0,b1);//name
        strList(bufp,',',1,b2);//E
        strList(bufp,',',2,b3);//poisson ratio
        dialogSetValue(dlg2,2,b1);
        dialogSetValue(dlg2,4,b2);
        dialogSetValue(dlg2,6,b3);
        loop3:
        w3=dialogExec(dlg2,"Edit Material");
        if(w3==7)//ok pressed
        {
            dialogGetValue(dlg2,4,b2);d1=atof(b2);//E
            if(d1<=0.0)
            {
              messageBox("Elastic Modulus cannot| be zero or negative.","Error");
              goto loop3;
            };
            dialogGetValue(dlg2,2,b1);//name
            if(strlen(b1)==0)
            {
              messageBox("Please enter a material name.","Error");
              goto loop2;
            };
            dialogGetValue(dlg2,6,b3);
            
	    sprintf(buf3,"%s,%s,%s",b1,b2,b3);//buf2 holds the edited material
            slistSet(l,w2,buf3);
        };
        goto loop1;
    }else if(w1==103)//Del
    {
        if(n==1)
        {
            messageBox("Cannot delete the last| material.","Error");
            goto loop1;
        };
        dialogGetValue(dlg1,1,buf3);w2=atoi(buf3);//w2 holds the current choice
        slistRemoveAt(l,w2);
	goto loop1;
    };
    slistFree(dlg1);dlg1=NULL;
    slistFree(dlg2);dlg2=NULL;
    return l;
}




//  AddIn_main (Sample program main function)
//  param   :   isAppli   : 1 = This application is launched by MAIN MENU.
//                        : 0 = This application is launched by a strip in eACT application.
//              OptionNum : Strip number (0~3)
//                         (This parameter is only used when isAppli parameter is 0.)
//  retval  :   1 = No error / 0 = Error

int AddIn_main(int isAppli, unsigned short OptionNum)
{
    unsigned int key;

	char buf1[512];
	char buf2[512];
	char* bufp=NULL;
	int w1,w2,w3,w4,w5,ret,i,j,k,l,g,h;
	int N,E,L;
	double d1,d2,d3,d4,d5,dx,dy,s,t,z,x,y,a,b,c,d;
	double eal,eil,eil2,eil3;
	struct Matrix* matN=NULL;//node input
	struct Matrix* matE=NULL;//elem input
	struct Matrix* matL=NULL;//load input
	struct Matrix* matG=NULL;//usage of nodes
	struct Matrix* matA=NULL;//global stiffness matrix
	struct Matrix* matB=NULL;//global load matrix
	struct Matrix* matF=NULL;//L,theta,dc,FEL,FEG ,FINAL
	struct Matrix* matR=NULL;//rotation of stiffness
	struct Matrix* matR1=NULL;
	struct Matrix* matU=NULL;
	struct Matrix* matK=NULL;//local elem stiffness
	struct Matrix* matK1=NULL;
	int* elemMaterials=NULL;
	int* elemSections=NULL;
	struct Viewport view;
    struct StringList* matList=NULL;
    struct StringList* sectList=NULL;
	struct StringList* dlg1=NULL;//dialog
	
	//init debug nums
	/*N=4;matN=matrixInit(matN,N,8);
	matrixSet(matN,1,1,0.0);matrixSet(matN,1,2,0.0);matrixSet(matN,1,3,1.0);matrixSet(matN,1,4,1.0);matrixSet(matN,1,5,1.0);
	matrixSet(matN,2,1,0.0);matrixSet(matN,2,2,5.0);matrixSet(matN,2,3,0.0);matrixSet(matN,2,4,0.0);matrixSet(matN,2,5,0.0);
	matrixSet(matN,3,1,8.0);matrixSet(matN,3,2,5.0);matrixSet(matN,3,3,0.0);matrixSet(matN,3,4,0.0);matrixSet(matN,3,5,0.0);
	matrixSet(matN,4,1,8.0);matrixSet(matN,4,2,0.0);matrixSet(matN,4,3,1.0);matrixSet(matN,4,4,1.0);matrixSet(matN,4,5,1.0);
	E=3;matE=matrixInit(matE,E,5);
	elemMaterials=(int *)malloc(sizeof(int)*(E+1));
	elemSections=(int *)malloc(sizeof(int)*(E+1));
		for(w1=1;w1<=E;w1++) {elemMaterials[w1]=1;elemSections[w1]=1;};
	matrixSet(matE,1,1, 1);matrixSet(matE,1,2, 2);matrixSet(matE,1,3,1);matrixSet(matE,1,4,1.0);matrixSet(matE,1,5,1);
	matrixSet(matE,2,1, 2);matrixSet(matE,2,2, 3);matrixSet(matE,2,3,1);matrixSet(matE,2,4,1.0);matrixSet(matE,2,5,1);
	matrixSet(matE,3,1, 3);matrixSet(matE,3,2, 4);matrixSet(matE,3,3,1);matrixSet(matE,3,4,1.0);matrixSet(matE,3,5,1);
	L=1;matL=matrixInit(matL,L,5);
	matrixSet(matL,1,1, 2);matrixSet(matL,1,2, 2);matrixSet(matL,1,3,0.0);matrixSet(matL,1,4,2.0);matrixSet(matL,1,5,0.0);
	*/
	
	N=0;E=0;L=0;
	matList=initMatList(matList);
    sectList=initSectList(sectList);
	mainmenu:
	while(1){
    Bdisp_AllClr_DDVRAM();
	PrintMini(1,25,"Beam v0.2",MINI_OVER);
	PrintMini(1,32,"(c) Toussis Manolis",MINI_OVER);
	PrintMini(1,39,"manolis @ ",MINI_OVER);
	PrintMini(1,46,"koppermind.homelinux.org",MINI_OVER);
	
	ret=execMenu("Node,Elem,Load,Solv,Data,Exit");
	if(ret==1)//nodes
	{
		if(matN==NULL)
		{
			locate(1,1);
			Print((unsigned char*)"Node num:");
			strInput(11,1,buf1,5,NULL,1);
			N=atoi(buf1);
			if(N<2) N=2;
			if(N>20) N=20;
			matN=matrixInit(matN,N,8);
		}else
		{
			w1=questionBox("Delete nodes?",0);
			Bdisp_AllClr_DDVRAM();
			if(w1==1)
			{
				locate(1,1);
				Print((unsigned char*)"Node num:");
				strInput(11,1,buf1,5,NULL,1);
				N=atoi(buf1);
				if(N<2) N=2;
				if(N>20) N=20;
				matrixFree(matN);
				if(matE!=NULL) {matrixFree(matE);matE=NULL;E=0;};
				if(matL!=NULL) {matrixFree(matL);matL=NULL;L=0;};
				matN=matrixInit(matN,N,8);
			};
		};
		w1=1;
		dlg1=slistInit(12,32);
	    slistSet(dlg1,1,"LABEL,1,8,X (m):");
		slistSet(dlg1,2,"EDIT,40,8,40,8,0.000,%.3f");
		slistSet(dlg1,3,"LABEL,1,18,Y (m):");
		slistSet(dlg1,4,"EDIT,40,18,40,8,0.000,%.3f");
		slistSet(dlg1,5,"CHECKBOX,1,26,ConX,0");
		slistSet(dlg1,6,"CHECKBOX,32,26,ConY,0");
		slistSet(dlg1,7,"CHECKBOX,62,26,ConT,0");
		slistSet(dlg1,8,"LABEL,1,37,Pnt loads|Presc.Displac (x-y-t):");
		slistSet(dlg1,9,"EDIT,2,47,40,8,0.000,%.3f");
		slistSet(dlg1,10,"EDIT,44,47,40,8,0.000,%.3f");
		slistSet(dlg1,11,"EDIT,86,47,40,8,0.000,%.3f");
		slistSet(dlg1,12,"MENU,PREV,NEXT,,,,DONE");
	   dlgJump1:
		//update the current node data to dialog
        sprintf(buf1,"%.3f",matrixGet(matN,w1,1));dialogSetValue(dlg1,2,buf1);
        sprintf(buf1,"%.3f",matrixGet(matN,w1,2));dialogSetValue(dlg1,4,buf1);
        d1=matrixGet(matN,w1,3);//x con
        if(d1==1.0) strcpy(buf1,"1"); else strcpy(buf1,"0");
        dialogSetValue(dlg1,5,buf1);
        d1=matrixGet(matN,w1,4);//y con
        if(d1==1.0) strcpy(buf1,"1"); else strcpy(buf1,"0");
        dialogSetValue(dlg1,6,buf1);
        d1=matrixGet(matN,w1,5);//t con
        if(d1==1.0) strcpy(buf1,"1"); else strcpy(buf1,"0");
        dialogSetValue(dlg1,7,buf1);
        sprintf(buf1,"%.3f",matrixGet(matN,w1,6));dialogSetValue(dlg1,9,buf1);
        sprintf(buf1,"%.3f",matrixGet(matN,w1,7));dialogSetValue(dlg1,10,buf1);
        sprintf(buf1,"%.3f",matrixGet(matN,w1,8));dialogSetValue(dlg1,11,buf1);
        //now exec the dialog
		sprintf(buf1,"Edit Node Number %d",w1);
		w2=dialogExec(dlg1,buf1);
		//update matN from dialog data
		dialogGetValue(dlg1,2,buf1);matrixSet(matN,w1,1,atof(buf1));
		dialogGetValue(dlg1,4,buf1);matrixSet(matN,w1,2,atof(buf1));
		dialogGetValue(dlg1,5,buf1);matrixSet(matN,w1,3,atoi(buf1));
		dialogGetValue(dlg1,6,buf1);matrixSet(matN,w1,4,atoi(buf1));
		dialogGetValue(dlg1,7,buf1);matrixSet(matN,w1,5,atoi(buf1));
		dialogGetValue(dlg1,9,buf1);matrixSet(matN,w1,6,atof(buf1));
		dialogGetValue(dlg1,10,buf1);matrixSet(matN,w1,7,atof(buf1));
		dialogGetValue(dlg1,11,buf1);matrixSet(matN,w1,8,atof(buf1));
		if(w2==101) //prev node
		{
            w1--;
            if(w1<1) w1=N;
            goto dlgJump1;
		}else if(w2==102) //next node
		{
            w1++;
            if(w1>N) w1=1;
            goto dlgJump1;
		}else if(w2==106) //done
		{
		};
		slistFree(dlg1);dlg1=NULL;
		//matrixEditor(matN,"X(m),Y(m),conX,conY,conT,pUx,pUy,pUt","  %.3f,  %.3f,    %.0f,    %.0f,    %.0f,%.5f,%.5f,%.5f");
	}else if(ret==2)//elems
	{
		if(N<2)
		{
            messageBox("There must be at least |2 nodes.","Error");
            goto mainmenu;
		};
		if(matE==NULL)
		{
			locate(1,1);
			Print((unsigned char*)"Elem num:");
			strInput(11,1,buf1,5,NULL,1);
			E=atoi(buf1);
			if(E<1 || E>20) E=1;
			matE=matrixInit(matE,E,5);
			elemMaterials=(int *)malloc(sizeof(int)*(E+1));
			elemSections=(int *)malloc(sizeof(int)*(E+1));
			for(w1=1;w1<=E;w1++) {elemMaterials[w1]=1;elemSections[w1]=1;};
			for(w1=1;w1<=2;w1++) for(w2=1;w2<=matE->rows;w2++) matrixSet(matE,w2,w1,1.0);
		}else
		{
			w1=questionBox("Delete elems?",0);
			Bdisp_AllClr_DDVRAM();
			if(w1==1)
			{
				locate(1,1);
				Print((unsigned char*)"Elem num:");
				strInput(11,1,buf1,5,NULL,1);
				E=atoi(buf1);
				if(E<1 || E>20) E=1;
				matrixFree(matE);
				if(matL!=NULL) {matrixFree(matL);matL=NULL;L=0;};
				free(elemMaterials);elemMaterials=NULL;
				free(elemSections);elemSections=NULL;
				matE=matrixInit(matE,E,5);
				elemMaterials=(int *)malloc(sizeof(int)*(E+1));
                elemSections=(int *)malloc(sizeof(int)*(E+1));
				for(w1=1;w1<=E;w1++) {elemMaterials[w1]=1;elemSections[w1]=1;};
				for(w1=1;w1<=2;w1++) for(w2=1;w2<=matE->rows;w2++) matrixSet(matE,w2,w1,1.0);
			};
		};
		w1=1;
		dlg1=slistInit(9,32);
	    slistSet(dlg1,1,"LABEL,1,8,node 1:");
        bufp=getNodeNames(N);
        sprintf(buf1,"DROPLIST,40,8,40,8,%d,%s,1",N,bufp);
        sprintf(buf2,"DROPLIST,40,18,40,8,%d,%s,1",N,bufp);
        free(bufp);
        slistSet(dlg1,2,buf1);
        slistSet(dlg1,3,"LABEL,1,18,node 2:");
	slistSet(dlg1,4,buf2);
        slistSet(dlg1,5,"LABEL,1,28,Material:");
        bufp=getListNames(matList);
        sprintf(buf1,"DROPLIST,40,28,70,8,%d,%s,1",matList->count,bufp);
        slistSet(dlg1,6,buf1);
        free(bufp);
        slistSet(dlg1,7,"LABEL,1,38,Section:");
        bufp=getListNames(sectList);
        sprintf(buf1,"DROPLIST,40,38,70,8,%d,%s,1",sectList->count,bufp);
        slistSet(dlg1,8,buf1);
        free(bufp);
        slistSet(dlg1,9,"MENU,PREV,NEXT,,,,DONE");
	   dlgJump2:
		//update the current element data
        sprintf(buf1,"%d",(int)matrixGet(matE,w1,1));
		dialogSetValue(dlg1,2,buf1);
        sprintf(buf1,"%d",(int)matrixGet(matE,w1,2));
		dialogSetValue(dlg1,4,buf1);
        sprintf(buf1,"%d",elemMaterials[w1]);
		dialogSetValue(dlg1,6,buf1);
        sprintf(buf1,"%d",elemSections[w1]);
		dialogSetValue(dlg1,8,buf1);
        //now exec the dialog
		sprintf(buf1,"Edit Elem Number %d",w1);
		w2=dialogExec(dlg1,buf1);
		//update matE from dialog data
		dialogGetValue(dlg1,2,buf1);i=atoi(buf1);//node i
		dialogGetValue(dlg1,4,buf1);j=atoi(buf1);//node j
		dialogGetValue(dlg1,6,buf1);k=atoi(buf1);//material
		dialogGetValue(dlg1,8,buf1);l=atoi(buf1);//section
		if(i==j)
		{
		  messageBox("Node 1 and Node 2 must| be different.","Error");
		  goto dlgJump2;
		};
		d1=getMaterialE(matList,k);
		d2=getSectionA(sectList,l);
		d3=getSectionI(sectList,l);
		matrixSet(matE,w1,1,i);
		matrixSet(matE,w1,2,j);
		elemMaterials[w1]=k;matrixSet(matE,w1,3,d1);
		elemSections[w1]=l;matrixSet(matE,w1,4,d3);matrixSet(matE,w1,5,d2);
		if(w2==101) //prev elem
		{
            w1--;
            if(w1<1) w1=E;
            goto dlgJump2;
		}else if(w2==102) //next node
		{
            w1++;
            if(w1>E) w1=1;
            goto dlgJump2;
		}else if(w2==106) //done
		{
		};
		slistFree(dlg1);dlg1=NULL;
	}else if(ret==3)//loads
	{
		if(E<1)
		{
            messageBox("There must be at least |1 element.","Error");
            goto mainmenu;
		};
		if(matL==NULL)
		{
			locate(1,1);
			Print((unsigned char *)"Load num:");
			strInput(11,1,buf1,5,NULL,1);
			L=atoi(buf1);
			if(L<0 || L>10) L=1;
			matL=matrixInit(matL,L,5);
			for(w1=1;w1<=L;w1++) {matrixSet(matL,w1,1,1.0);matrixSet(matL,w1,2,1.0);};
		}else
		{
			w1=questionBox("Delete loads?",0);
			Bdisp_AllClr_DDVRAM();
			if(w1==1)
			{
				locate(1,1);
				Print((unsigned char *)"Load num:");
				strInput(11,1,buf1,5,NULL,1);
				L=atoi(buf1);
				if(L<0 || L>10) L=1;
				matrixFree(matL);
				matL=matrixInit(matL,L,5);
				for(w1=1;w1<=L;w1++) {matrixSet(matL,w1,1,1.0);matrixSet(matL,w1,2,1.0);};
			};
		};
		
		w1=1;
		dlg1=slistInit(9,32);
	    slistSet(dlg1,1,"LABEL,1,8,element:");
        bufp=getElementNames(E);
        sprintf(buf1,"DROPLIST,40,8,40,8,%d,%s,1",E,bufp);
        free(bufp);
        slistSet(dlg1,2,buf1);
        slistSet(dlg1,3,"LABEL,1,18,load type:");
	slistSet(dlg1,4,"DROPLIST,45,18,70,8, 2,Point load,Uniform load,1");
	slistSet(dlg1,5,"LABEL,1,28,x(meters):");
        slistSet(dlg1,6,"EDITBOX,45,28,50,8,0.00,%.2f");
	slistSet(dlg1,7,"LABEL,1,38,b(kN or kN/m):");
	slistSet(dlg1,8,"EDITBOX,65,38,50,8,0.000,%.3f");
	slistSet(dlg1,9,"MENU,PREV,NEXT,,,,DONE");
	   dlgJump3:
		//update the current load data
        sprintf(buf1,"%d",(int)matrixGet(matL,w1,1));
		dialogSetValue(dlg1,2,buf1);
        sprintf(buf1,"%d",(int)matrixGet(matL,w1,2));
		dialogSetValue(dlg1,4,buf1);
        sprintf(buf1,"%f",matrixGet(matL,w1,3));
		dialogSetValue(dlg1,6,buf1);
        sprintf(buf1,"%f",matrixGet(matL,w1,4));
		dialogSetValue(dlg1,8,buf1);
        //now exec the dialog
		sprintf(buf1,"Edit Load Number %d",w1);
		w2=dialogExec(dlg1,buf1);
		//update matL from dialog data
		dialogGetValue(dlg1,2,buf1);i=atoi(buf1);//load element
		dialogGetValue(dlg1,4,buf1);j=atoi(buf1);//load type
		dialogGetValue(dlg1,6,buf1);d1=atof(buf1);//a(m)
		dialogGetValue(dlg1,8,buf1);d2=atof(buf1);//b(kN)
		matrixSet(matL,w1,1,i);
		matrixSet(matL,w1,2,j);
		matrixSet(matL,w1,3,d1);
		matrixSet(matL,w1,4,d2);
		if(w2==101) //prev load
		{
            w1--;
            if(w1<1) w1=L;
            goto dlgJump3;
		}else if(w2==102) //next load
		{
            w1++;
            if(w1>L) w1=1;
            goto dlgJump3;
		}else if(w2==106) //done
		{
		};
		slistFree(dlg1);dlg1=NULL;		
		//matrixEditor(matL,"el.nod,1P.2q,a(x),b(Pq),c","    %.0f,    %.0f,%.4f,%.4f,%.4f");
	}else if(ret==4)//solve
	{
		locate(1,1);
		Print((unsigned char *)"Init system...");
		if(N<2 || E<1)
		{
			messageBox("There must be at least|2 nodes and 1 element.","Error");
			goto mainmenu;
		};
		if(matA!=NULL) matrixFree(matA);
		if(matB!=NULL) matrixFree(matB);
		matA=matrixInit(matA,N*3,N*3);
		matB=matrixInit(matB,N*3,1);
		if(matF!=NULL) matrixFree(matF);
		matF=matrixInit(matF,E,21);
		if(matG!=NULL) matrixFree(matG);
		matG=matrixInit(matG,N,1);
		for(k=1;k<=E;k++)
		{
			w1=(int)matrixGet(matE,k,1);
			matrixSet(matG,w1,1, matrixGet(matG,w1,1)+1);
			w1=(int)matrixGet(matE,k,2);
			matrixSet(matG,w1,1, matrixGet(matG,w1,1)+1);
		};
		for(k=1;k<=E;k++)
		{
			w1=(int)matrixGet(matE,k,1);
			w2=(int)matrixGet(matE,k,2);
			if(matrixGet(matG,w1,1)<=1.0 && matrixGet(matN,w1,5)==0.0) g=0; else g=1;//left
			if(matrixGet(matG,w2,1)<=1.0 && matrixGet(matN,w2,5)==0.0) h=0; else h=1;//right
			if(g==0 && h==1) matrixSet(matF,k,3, 2); //right end
				else if(g==1 && h==0) matrixSet(matF,k,3, 1);//left end
				else matrixSet(matF,k,3, 0);//both ends
		};
		matrixFree(matG);matG=NULL;
		//calculate L,theta
		for(k=1;k<=E;k++)
		{
			w1=(int)matrixGet(matE,k,1);
			w2=(int)matrixGet(matE,k,2);
			dx=matrixGet(matN,w2,1)-matrixGet(matN,w1,1);
			dy=matrixGet(matN,w2,2)-matrixGet(matN,w1,2);
			d1=sqrt(dx*dx+dy*dy);//element length
			if(d1<=0.0)
			{
				sprintf(buf1,"Element %d has zero length.",k);
				messageBox(buf1,"Error");
				goto mainmenu;
			};
			if(w1==w2)
			{
				sprintf(buf1,"Element %d has node %d as|both start and end.",k,w1);
				messageBox(buf1,"Error");
				goto mainmenu;
			};
			matrixSet(matF,k,1,d1);
			d1=0.0;
			if(dx==0.0 && dy<0.0) d1=3.0*PI/2.0;
				else if(dx==0.0 && dy>0.0) d1=PI/2.0;
				else if(dx>0.0 && dy>0.0) d1=atan(dy/dx);
				else if(dx>0.0 && dy<0.0) d1=2.0*PI+atan(dy/dx);
				else if(dx<0.0) d1=PI+atan(dy/dx);
			matrixSet(matF,k,2, d1);
		};
		//now calculate fixed end local 4-9 , fixed end global 10-15
		locate(1,2);
		Print((unsigned char *)"Computing fixed ends.");
		for(k=1;k<=L;k++)
		{
			i=(int)matrixGet(matL,k,1);//element number
			x=matrixGet(matL,k,3);//a
			y=matrixGet(matL,k,4);//b
			s=matrixGet(matF,i,1);//length of member
			w3=(int)matrixGet(matL,k,2);//type of load
			if(w3==1)//point load
			{
				g=(int)matrixGet(matF,i,3);
				if(g==0)// amfipakto
				{
					d1=matrixGet(matF,i,5) - (x-s)*(x-s)*(2*x+s)/(s*s*s)*y;
					matrixSet(matF,i,5, d1);//Qa
					d1=matrixGet(matF,i,6) + y*x*((s-x)/s*(s-x)/s);
					matrixSet(matF,i,6, d1);//Ma
					d1=matrixGet(matF,i,8) - x*x*(-2*x+3*s)/(s*s*s)*y;
					matrixSet(matF,i,8, d1);//Qb
					d1=matrixGet(matF,i,9) - y*(s-x)*(x/s)*(x/s);
					matrixSet(matF,i,9, d1);//Mb
				}else if(g==1)// monopakto aristera
				{
					d1=matrixGet(matF,i,5) - (x*x*x-3*x*x*s+2*s*s*s)/(2*s*s*s)*y; //Qa
					matrixSet(matF,i,5, d1);
					d1=matrixGet(matF,i,6) + y*x*(s-x)/(2*s)*(1+(s-x)/s);//Ma
					matrixSet(matF,i,6, d1);
					d1=matrixGet(matF,i,8) - (-x*x*(x-3*s)/(2*s*s*s)*y);//Qb
					matrixSet(matF,i,8, d1);
				}else if(g==2)//monopakto deksia
				{
					d1=matrixGet(matF,i,5) - (-x*x*(x-3*s)/(2*s*s*s)*y);//Qa
					matrixSet(matF,i,5, d1);
					d1=matrixGet(matF,i,8) - (x*x*x-3*x*x*s+2*s*s*s)/(2*s*s*s)*y;//Qb
					matrixSet(matF,i,8, d1);
					d1=matrixGet(matF,i,9) - y*x*(s-x)/(2*s)*(1+(s-x)/s);//Mb
					matrixSet(matF,i,9, d1);
				};
			}//end point load
			else if(w3==2)//uniform load
			{
				g=(int)matrixGet(matF,i,3);
				if(g==0)// amfipakto
				{
					d1=matrixGet(matF,i,5) - y*s/2.0;//Qa
					matrixSet(matF,i,5, d1);
					d1=matrixGet(matF,i,6) + y*s*s/12.0;//Ma
					matrixSet(matF,i,6, d1);
					d1=matrixGet(matF,i,8) - y*s/2.0;//Qb
					matrixSet(matF,i,8, d1);
					d1=matrixGet(matF,i,9) - y*s*s/12.0;//Mb
					matrixSet(matF,i,9, d1);
				}else if(g==1)// monopakto aristera
				{
					d1=matrixGet(matF,i,5) - 5*y*s/8.0;//Qa
					matrixSet(matF,i,5, d1);
					d1=matrixGet(matF,i,6) + y*s*s/8.0;//Ma
					matrixSet(matF,i,6, d1);
					d1=matrixGet(matF,i,8) - 3*y*s/8.0;//Qb
					matrixSet(matF,i,8, d1);
				}else if(g==2)//monopakto deksia
				{
					d1=matrixGet(matF,i,5) - 3*y*s/8.0;//Qa
					matrixSet(matF,i,5, d1);
					d1=matrixGet(matF,i,8) - 5*y*s/8.0;//Qb
					matrixSet(matF,i,8, d1);
					d1=matrixGet(matF,i,9) - y*s*s/8.0;//Mb
					matrixSet(matF,i,9, d1);
				};
			};//end of uniform load
		};//end of fixed end locals
		//start of fixed end globals
		for(k=1;k<=E;k++)
		{
			x=sin(matrixGet(matF,k,2));
			y=cos(matrixGet(matF,k,2));
			d1=y*matrixGet(matF,k,4) - x*matrixGet(matF,k,5);
			matrixSet(matF,k,10, d1);
			d1=x*matrixGet(matF,k,4) + y*matrixGet(matF,k,5);
			matrixSet(matF,k,11, d1);
		    d1=matrixGet(matF,k,6);
			matrixSet(matF,k,12, d1);
			d1=y*matrixGet(matF,k,7) - x*matrixGet(matF,k,8);
			matrixSet(matF,k,13, d1);
			d1=x*matrixGet(matF,k,7) + y*matrixGet(matF,k,8);
			matrixSet(matF,k,14, d1);
		    d1=matrixGet(matF,k,9);
			matrixSet(matF,k,15, d1);
		};//end of fixed end globals
		//now assemble stiffness matrix;
		locate(1,3);
		Print((unsigned char *)"Stiffness matrix.");
		matK=matrixInit(matK,6,6);
		matR=matrixInit(matR,6,6);
		for(k=1;k<=E;k++)
		{
			s=matrixGet(matF,k,1);//length
			t=matrixGet(matF,k,2);//theta
			eal=matrixGet(matE,k,3)*matrixGet(matE,k,5)/s; //EA/L
			eil=matrixGet(matE,k,3)*matrixGet(matE,k,4)/s; //EI/L
			eil2=eil / s; //EI/L^2
			eil3=eil2 / s; //EI/L^3
			matrixZero(matK);
			matrixSet(matK,1,1,eal);matrixSet(matK,1,4,-eal);
			matrixSet(matK,2,2,12*eil3);matrixSet(matK,2,3,-6*eil2);matrixSet(matK,2,5,-12*eil3);matrixSet(matK,2,6,-6*eil2);
			matrixSet(matK,3,2,-6*eil2);matrixSet(matK,3,3,4*eil);matrixSet(matK,3,5,6*eil2);matrixSet(matK,3,6,2*eil);
			matrixSet(matK,4,1,-eal);matrixSet(matK,4,4,eal);
			matrixSet(matK,5,2,-12*eil3);matrixSet(matK,5,3,6*eil2);matrixSet(matK,5,5,12*eil3);matrixSet(matK,5,6,6*eil2);
			matrixSet(matK,6,2,-6*eil2);matrixSet(matK,6,3,2*eil);matrixSet(matK,6,5,6*eil2);matrixSet(matK,6,6,4*eil);
			x=sin(t);
			y=cos(t);
			matrixZero(matR);
			matrixSet(matR,1,1,y);matrixSet(matR,1,2,x);
			matrixSet(matR,2,1,-x);matrixSet(matR,2,2,y);
			matrixSet(matR,3,3,1.0);
			matrixSet(matR,4,4,y);matrixSet(matR,4,5,x);
			matrixSet(matR,5,4,-x);matrixSet(matR,5,5,y);
			matrixSet(matR,6,6,1.0);
			matR1=matrixTranspose(matR); // Kglobal = Transpose(MatR) x Klocal x MatR
			matK1=matrixMulMatrix(matR1,matK);
			matrixFree(matR1);matR1=NULL;
			matR1=matrixMulMatrix(matK1,matR);
			matrixFree(matK1);//matR1 holds the Kglobal
			//now put matR1 to matA (global stiffness)
			//and matF global to matB
			matK1=matrixInit(matK1,6,1);//6 dofs
			i=(int)matrixGet(matE,k,1);//node i
			j=(int)matrixGet(matE,k,2);//node j
			matrixSet(matK1,1,1, (i-1)*3+1);
			matrixSet(matK1,2,1, (i-1)*3+2);
			matrixSet(matK1,3,1, (i-1)*3+3);
			matrixSet(matK1,4,1, (j-1)*3+1);
			matrixSet(matK1,5,1, (j-1)*3+2);
			matrixSet(matK1,6,1, (j-1)*3+3);
			for(i=1;i<=6;i++)
			{
				w1=matrixGet(matK1,i,1);
				for(j=1;j<=6;j++)
				{
					w2=matrixGet(matK1,j,1);
					d1=matrixGet(matA,w1,w2)+matrixGet(matR1,i,j);
					matrixSet(matA,w1,w2,d1);
				};
				d1=matrixGet(matB,w1,1)+matrixGet(matF,k,10+(i-1));
				matrixSet(matB,w1,1,d1);
			};
			matrixFree(matK1);matK1=NULL;
			matrixFree(matR1);matR1=NULL;
		};
		matrixFree(matK);matK=NULL;
		matrixFree(matR);matR=NULL;
		//node loads
		for(k=1;k<=N;k++)
		{
			if(matrixGet(matN,k,3)==0.0 && matrixGet(matN,k,6)!=0.0)//X point load
			{
				i=(k-1)*3+1;
				matrixSet(matB,i,1,matrixGet(matB,i,1)+matrixGet(matN,k,6));
			};
			if(matrixGet(matN,k,4)==0.0 && matrixGet(matN,k,7)!=0.0)//Y point load
			{
				i=(k-1)*3+2;
				matrixSet(matB,i,1,matrixGet(matB,i,1)+matrixGet(matN,k,7));
			};
			if(matrixGet(matN,k,5)==0.0 && matrixGet(matN,k,8)!=0.0)//rZ point load
			{
				i=(k-1)*3+3;
				matrixSet(matB,i,1,matrixGet(matB,i,1)+matrixGet(matN,k,8));
			};
		};


		//add constraints
		locate(1,4);
		Print((unsigned char *)"Add constraints.");
		for(i=1;i<=N*3;i++)//dof
		{
			k=(i-1)/3+1;//node number
			j=(i-1)%3;//local dof 0,1,2
			if(matrixGet(matN,k,3+j)!=0.0)// if reaction
			{
				matrixSet(matB,i,1, matrixGet(matN,k,6+j));
			}else //not reaction
			{
				for(w1=1;w1<=N*3;w1++)
				{
					w2=(w1-1)/3+1;
					w3=(w1-1)%3;
					if(matrixGet(matN,w2,3+w3)!=0.0)
					{
						d1=matrixGet(matA,i,w1)*matrixGet(matN,w2,6+w3);//w2=w1???
						matrixSet(matB,i,1, matrixGet(matB,i,1)-d1);
					};
				};
			}
		};
		for(i=1;i<=N*3;i++)//dof
		{
			k=(i-1)/3+1;//node number
			j=(i-1)%3;//local dof 0,1,2
			if(matrixGet(matN,k,3+j)!=0.0)//if reaction
			{
				for(w1=1;w1<=N*3;w1++)
				{
					if(i==w1) matrixSet(matA,i,w1,1.0);
						else {matrixSet(matA,i,w1,0.0);matrixSet(matA,w1,i,0.0);};
				};
			};
		};
		//d1=matrixDeterminant(matA);
		//sprintf(buf1,"det:%f",d1);
		//messageBox(buf1,"Warning");
		//TODO check determinant
		//matrixEditor(matA,"",NULL);
		//matrixEditor(matB,"",NULL);
		/*for(k=1;k<=N;k++)
		{
			if(matrixGet(matN,k,3)==1.0)//N constraint
			{
				i=(k-1)*3+1;
				for(j=1;j<=N*3;j++)
				{
					matrixSet(matA,i,j, 0.0);
					matrixSet(matA,j,i, 0.0);
				};
				matrixSet(matA,i,i, 1.0);
				d1=matrixGet(matN,k,6);
				matrixSet(matB,i,1, d1);
			};
			if(matrixGet(matN,k,4)==1.0)//Q constraint
			{
				i=(k-1)*3+2;
				for(j=1;j<=N*3;j++)
				{
					matrixSet(matA,i,j, 0.0);
					matrixSet(matA,j,i, 0.0);
				};
				matrixSet(matA,i,i, 1.0);
				d1=matrixGet(matN,k,7);
				matrixSet(matB,i,1, d1);
			};
			if(matrixGet(matN,k,5)==1.0)//M constraint
			{
				i=(k-1)*3+3;
				for(j=1;j<=N*3;j++)
				{
					matrixSet(matA,i,j, 0.0);
					matrixSet(matA,j,i, 0.0);
				};
				matrixSet(matA,i,i, 1.0);
				d1=matrixGet(matN,k,8);
				matrixSet(matB,i,1, d1);
			};
		};*/
		//solve system
		locate(1,5);
		Print((unsigned char *)"Solve system.");
		matB=matrixLinearSolve(matA, matB);
		//matB=matrixConjGradient(matA, matB,1.0E-25);
		if(matB==NULL)
		{
			messageBox("Unstable system of equations.","Error");
			goto mainmenu;
		};
		matrixFree(matA);matA=NULL;//matB holds the solution
		//calculate local element forces
		locate(1,6);
		Print((unsigned char *)"Local Elem Forces");
		matK1=matrixInit(matK1,6,1);//6 dofs
		matK=matrixInit(matK,6,6);//K local
		matU=matrixInit(matU,6,1);//6 displacements
		for(k=1;k<=E;k++)
		{
			//compute matUlocal
			i=(int)matrixGet(matE,k,1);
			j=(int)matrixGet(matE,k,2);
			matrixSet(matK1,1,1, (i-1)*3+1);
			matrixSet(matK1,2,1, (i-1)*3+2);
			matrixSet(matK1,3,1, (i-1)*3+3);
			matrixSet(matK1,4,1, (j-1)*3+1);
			matrixSet(matK1,5,1, (j-1)*3+2);
			matrixSet(matK1,6,1, (j-1)*3+3);
			x=sin(matrixGet(matF,k,2));
			y=cos(matrixGet(matF,k,2));
			s=matrixGet(matB,(int)matrixGet(matK1,1,1),1);
			t=matrixGet(matB,(int)matrixGet(matK1,2,1),1);
			matrixSet(matU,1,1, y*s+x*t);
			matrixSet(matU,2,1, -x*s+y*t);
			matrixSet(matU,3,1, matrixGet(matB,(int)matrixGet(matK1,3,1),1));
			s=matrixGet(matB,(int)matrixGet(matK1,4,1),1);
			t=matrixGet(matB,(int)matrixGet(matK1,5,1),1);
			matrixSet(matU,4,1, y*s+x*t);
			matrixSet(matU,5,1, -x*s+y*t);
			matrixSet(matU,6,1, matrixGet(matB,(int)matrixGet(matK1,6,1),1));
			//compute local stiffness
			s=matrixGet(matF,k,1);//length
			t=matrixGet(matF,k,2);//theta
			eal=matrixGet(matE,k,3)*matrixGet(matE,k,5)/s; //EA/L
			eil=matrixGet(matE,k,3)*matrixGet(matE,k,4)/s; //EI/L
			eil2=eil / s; //EI/L^2
			eil3=eil2 / s; //EI/L^3
			matrixZero(matK);
			matrixSet(matK,1,1,eal);matrixSet(matK,1,4,-eal);
			matrixSet(matK,2,2,12*eil3);matrixSet(matK,2,3,-6*eil2);matrixSet(matK,2,5,-12*eil3);matrixSet(matK,2,6,-6*eil2);
			matrixSet(matK,3,2,-6*eil2);matrixSet(matK,3,3,4*eil);matrixSet(matK,3,5,6*eil2);matrixSet(matK,3,6,2*eil);
			matrixSet(matK,4,1,-eal);matrixSet(matK,4,4,eal);
			matrixSet(matK,5,2,-12*eil3);matrixSet(matK,5,3,6*eil2);matrixSet(matK,5,5,12*eil3);matrixSet(matK,5,6,6*eil2);
			matrixSet(matK,6,2,-6*eil2);matrixSet(matK,6,3,2*eil);matrixSet(matK,6,5,6*eil2);matrixSet(matK,6,6,4*eil);
			matR1=matrixMulMatrix(matK,matU); //Klocal x ulocal
			//now add external loads
			matrixSet(matR1,1,1, -matrixGet(matR1,1,1)+matrixGet(matF,k,4));
			matrixSet(matR1,2,1, matrixGet(matR1,2,1)-matrixGet(matF,k,5));
			matrixSet(matR1,3,1, matrixGet(matR1,3,1)-matrixGet(matF,k,6));
			matrixSet(matR1,4,1, matrixGet(matR1,4,1)-matrixGet(matF,k,7));
			matrixSet(matR1,5,1, -matrixGet(matR1,5,1)+matrixGet(matF,k,8));
			matrixSet(matR1,6,1, -matrixGet(matR1,6,1)+matrixGet(matF,k,9));
			matrixSet(matF,k,16, matrixGet(matR1,1,1));
			matrixSet(matF,k,17, matrixGet(matR1,2,1));
			matrixSet(matF,k,18, matrixGet(matR1,3,1));
			matrixSet(matF,k,19, matrixGet(matR1,4,1));
			matrixSet(matF,k,20, matrixGet(matR1,5,1));
			matrixSet(matF,k,21, matrixGet(matR1,6,1));
			matrixFree(matR1);matR1=NULL;
		};
		matrixFree(matK1);matrixFree(matK);matrixFree(matU);
		matK1=NULL;matK=NULL;matU=NULL;

		//calculate external reactions
		matR1=matrixInit(matR1,N,6);//3 external reactions 3 global displacements for each node
		for(k=1;k<=E;k++)
		{
			i=(int)matrixGet(matE,k,1);
			j=(int)matrixGet(matE,k,2);
			x=sin(matrixGet(matF,k,2));
			y=cos(matrixGet(matF,k,2));
			if(matrixGet(matN,i,3)!=0.0)
			{
			d1=-y*matrixGet(matF,k,16)-x*matrixGet(matF,k,17);
			matrixSet(matR1,i,1, matrixGet(matR1,i,1)+d1);
			};
			if(matrixGet(matN,i,4)!=0.0)
			{
			d1=x*matrixGet(matF,k,16)-y*matrixGet(matF,k,17);
			matrixSet(matR1,i,2, matrixGet(matR1,i,2)+d1);
			};
			if(matrixGet(matN,i,5)!=0.0)
			{
			d1=-matrixGet(matF,k,18);
			matrixSet(matR1,i,3, matrixGet(matR1,i,3)+d1);
			};

			if(matrixGet(matN,j,3)!=0.0)
			{
			d1=-y*matrixGet(matF,k,19)+x*matrixGet(matF,k,20);
			matrixSet(matR1,j,1, matrixGet(matR1,j,1)+d1);
			};
			if(matrixGet(matN,j,4)!=0.0)
			{
			d1=-x*matrixGet(matF,k,19)+y*matrixGet(matF,k,20);
			matrixSet(matR1,j,2, matrixGet(matR1,j,2)+d1);
			};
			if(matrixGet(matN,j,5)!=0.0)
			{
			d3=-matrixGet(matF,k,21);
			matrixSet(matR1,j,3, matrixGet(matR1,j,3)+d3);
			};
			//global displacements
			matrixSet(matR1,i,4, matrixGet(matB,(i-1)*3+1,1));
			matrixSet(matR1,i,5, matrixGet(matB,(i-1)*3+2,1));
			matrixSet(matR1,i,6, matrixGet(matB,(i-1)*3+3,1));
			matrixSet(matR1,j,4, matrixGet(matB,(j-1)*3+1,1));
			matrixSet(matR1,j,5, matrixGet(matB,(j-1)*3+2,1));
			matrixSet(matR1,j,6, matrixGet(matB,(j-1)*3+3,1));
		};
		matrixFree(matB);matB=NULL;
		g=1;//1 elems 2 nodes
		k=1;//first elem
		//now matB holds the disps, matR1 the node Reaction, matF 16-21 the inside forces
		while(1)
		{
			Bdisp_AllClr_DDVRAM();
			if(g==1)//elems
			{
				i=(int)matrixGet(matE,k,1);//n1
				j=(int)matrixGet(matE,k,2);//n2
				sprintf(buf1,"Elem:%d n1:%d n2:%d",k,i,j);
				locate(1,1);
				Print((unsigned char*)buf1);
				sprintf(buf1,"Nn1:%f",matrixGet(matF,k,16));
				locate(1,2);
				Print((unsigned char*)buf1);
				sprintf(buf1,"Qn1:%f",matrixGet(matF,k,17));
				locate(1,3);
				Print((unsigned char*)buf1);
				sprintf(buf1,"Mn1:%f",matrixGet(matF,k,18));
				locate(1,4);
				Print((unsigned char*)buf1);
				sprintf(buf1,"Nn2:%f",matrixGet(matF,k,19));
				locate(1,5);
				Print((unsigned char*)buf1);
				sprintf(buf1,"Qn2:%f",matrixGet(matF,k,20));
				locate(1,6);
				Print((unsigned char*)buf1);
				sprintf(buf1,"Mn2:%f",matrixGet(matF,k,21));
				locate(1,7);
				Print((unsigned char*)buf1);
			}else if(g==2)//nodes
			{
				sprintf(buf1,"Node:%d",k);
				locate(1,1);
				Print((unsigned char*)buf1);
				sprintf(buf1,"Ux:%f",matrixGet(matR1,k,4));
				locate(1,2);
				Print((unsigned char*)buf1);
				sprintf(buf1,"Uy:%f",matrixGet(matR1,k,5));
				locate(1,3);
				Print((unsigned char*)buf1);
				sprintf(buf1,"Ut:%f",matrixGet(matR1,k,6));
				locate(1,4);
				Print((unsigned char*)buf1);
				w1=5;
				if(matrixGet(matN,k,3)!=0.0)
				{
				locate(1,w1);
				d1=matrixGet(matR1,k,1);
				sprintf(buf1,"Rx:%f",d1);
				Print((unsigned char*)buf1);
				w1++;
				};
				if(matrixGet(matN,k,4)!=0.0)
				{
				locate(1,w1);
				d1=matrixGet(matR1,k,2);
				sprintf(buf1,"Ry:%f",d1);
				Print((unsigned char*)buf1);
				w1++;
				};
				if(matrixGet(matN,k,5)!=0.0)
				{
				locate(1,w1);
				d1=matrixGet(matR1,k,3);
				sprintf(buf1,"Rt:%f",d1);
				Print((unsigned char*)buf1);
				w1++;
				};
			};
			w4=execMenu("PrvE,NxtE,PrvN,NxtN,View,Done");
			if(w4==2)
			{
				k++;
				if(k>E) k=1;
				if(g==2) {k=1;g=1;};
			}else if(w4==1)
			{
				k--;
				if(k<1) k=E;
				if(g==2) {k=1;g=1;};
			}else if(w4==3)
			{
				k--;
				if(k<1) k=N;
				if(g==1) {k=1;g=2;};
			}else if(w4==4)
			{
				k++;
				if(k>N) k=1;
				if(g==1) {k=1;g=2;};
			}else if(w4==5)
			{

				h=3;//M
				if(g==1) while(1)
				{
				loopdraw:
					Bdisp_AllClr_DDVRAM();
					i=(int)matrixGet(matE,k,1);//nodes
					j=(int)matrixGet(matE,k,2);
					s=matrixGet(matF,k,1);//element length
					d1=matrixGet(matN,i,1);d2=matrixGet(matN,i,2);//x1,y1
					d3=matrixGet(matN,j,1);d4=matrixGet(matN,j,2);//x2,y2
					view.leftX=0.0;view.rightX=1.0;view.bottomY=-1.0;view.topY=1.0;
					viewScale(1.5,&view);
					viewLine(0.0,0.0,1.0,0.0,&view);
					sprintf(buf1,"n%d",i);
					PrintMini(10,30,(unsigned char*)buf1,MINI_OVER);
					sprintf(buf1,"n%d",j);
					PrintMini(110,30,(unsigned char*)buf1,MINI_OVER);
					sprintf(buf1,"e%d",k);
					PrintMini(60,35,(unsigned char*)buf1,MINI_OVER);
					y=1E-10;//find maxM,Q,N
					for(w2=1;w2<=11;w2++)
					{
						x=(w2-1)/10.0*s;
						if(h==3) d5=getM(x,k,matF,matL,L);
							else if(h==2) d5=getQ(x,k,matF,matL,L);
							else if(h==1) d5=getN(x,k,matF,matL,L);
						if(fabs(d5)>y) y=fabs(d5);
					};
					z=1.0/y;//scale factor
					a=0.0;
					b=0.0;
					for(w2=1;w2<=101;w2++)
					{
						x=(w2-1)/100.0;
						if(h==3) d5=-getM(x*s,k,matF,matL,L);
							else if(h==2) d5=getQ(x*s,k,matF,matL,L);
							else if(h==1) d5=getN(x*s,k,matF,matL,L);
						c=x;
						d=d5*z;
						viewLine(a,b,c,d,&view);
						if((w2-1)%5==0) viewPerpendicularLine(0.0,0.0,1.0,0.0,x,d5*z,&view);
						a=c;b=d;
					};
					sprintf(buf1,"%f",matrixGet(matF,k,15+h));
					PrintMini(0,20,(unsigned char*)buf1,MINI_OVER);
					sprintf(buf1,"%f",matrixGet(matF,k,18+h));
					PrintMini(90,20,(unsigned char*)buf1,MINI_OVER);
					if(h==3) PrintMini(50,0,(unsigned char*)"M Diagram",MINI_OVER);
						else if(h==2) PrintMini(50,0,(unsigned char*)"Q Diagram",MINI_OVER);
						else if(h==1) PrintMini(50,0,(unsigned char*)"N Diagram",MINI_OVER);
					w5=execMenu(" M, Q, N,PrvE,NxtE,Done");
					if(w5==1)//M
					{
						h=3;
					}else if(w5==2)//Q
					{
						h=2;
					}else if(w5==3)//N
					{
						h=1;
					}else if(w5==4)//prvElement
					{
						k--;if(k<1) k=E;
					}else if(w5==5)//nxtElement
					{
						k++;if(k>E) k=1;
					}else if(w5==6)
					{
						break;
					};

				};
			}else if(w4==6)
			{
				matrixFree(matR1);matR1=NULL;
				matrixFree(matF);matF=NULL;
				break;
			};
		};
	}else if(ret==5)//material data section data
	{
        w1=execMenu("Mat,Sect,,,,Done");
        if(w1==1)
        {
            matList=materialEdit(matList);
        }else if(w1==2)
        {
            sectList=sectionEdit(sectList);
        }else if(w1==6)
        {

        };

	}else if(ret==6)//end program
	{
		if(matN!=NULL) {matrixFree(matN);matN=NULL;};
		if(matE!=NULL) {matrixFree(matE);matE=NULL;};
		if(matL!=NULL) {matrixFree(matL);matL=NULL;};
		if(matA!=NULL) {matrixFree(matA);matA=NULL;};
		if(matB!=NULL) {matrixFree(matB);matB=NULL;};
		if(matF!=NULL) {matrixFree(matF);matF=NULL;};
		if(elemMaterials!=NULL) free(elemMaterials);
		if(elemSections!=NULL) free(elemSections);
		break;
	};
	};//end while

    matList=saveMatList(matList);
    sectList=saveSectList(sectList);

    return 1;
}

#pragma section _BR_Size
unsigned long BR_Size;
#pragma section


#pragma section _TOP

int InitializeSystem(int isAppli, unsigned short OptionNum)
{
    return INIT_ADDIN_APPLICATION(isAppli, OptionNum);
}

#pragma section

