/*
Beam-Frame Structural Analysis for Casio 9860SDK version 0.2
Copyright (C) 2011 Toussis Manolis (koppermind at gmail.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "mylib.h"


#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "fxlib.h"

#define CHARXS	609/100
#define CHARYS	800/100

char debugB[2048]; //debugBuf
unsigned int debugkey; 
unsigned char* debugScr;


int GetFreeMemory(void)
{
  int i;
  Bfile_GetMediaFree(DEVICE_MAIN_MEMORY,&i);
  return(i);
}


/*DOC: gets or sets the screen contents in a pointer(malloc happens):
unsigned char* p;
p=GetScreen();
blah blah blah
SetScreen(p);
free(p);
*/
unsigned char* GetScreen(void)
{
  unsigned char* p;
  p=(unsigned char *)malloc(1024); //1024 bytes size
  ASSERT(p!=NULL);
  Bdisp_PutDisp_DD();
  Bdisp_GetDisp_DD(p);
  return(p);
}


void SetScreen(unsigned char* p)
{
  GRAPHDATA g;
  DISPGRAPH d;
  g.height=64;g.width=128;g.pBitmap=p;
  d.x=0;d.y=0;d.GraphData=g;d.WriteModify=IMB_WRITEMODIFY_NORMAL;d.WriteKind=IMB_WRITEKIND_OVER;
  Bdisp_WriteGraph_DDVRAM(&d);
}

/*DOC: StringList linked list
*/
struct StringListNode* slistCreateNode(const char* text)
{
  struct StringListNode* item;
  int len;
  item=malloc(sizeof(struct StringListNode));
  ASSERT(item!=NULL);
  len=strlen(text)+1;
  item->data=(char *)malloc(sizeof(char)*len);
  ASSERT(item->data!=NULL);
  item->maxCount=len;
  item->next=NULL;
  strcpy(item->data,text);
  return item;
}

void slistPrepend(struct StringList* list,const char* text)
{
  struct StringListNode* item;
  struct StringListNode* tmp;
  item=slistCreateNode(text);
  tmp=list->first;
  list->first=item;
  item->next=tmp; 
  list->count++;
}

struct StringList* slistInit(int n,int itemsize)
//inits list with m element of maxsize=itemsize
{
  struct StringList* list;
  struct StringListNode* item;
  char* bufp;
  int w1;
  list=malloc(sizeof(struct StringList));
  ASSERT(list!=NULL);
  list->first=NULL;
  list->count=0;
  bufp=(char *)malloc(sizeof(char)*(itemsize+1));
  ASSERT(bufp!=NULL);
  memset(bufp,32,itemsize);//fill with space(ascii 32)
  bufp[itemsize]=0;//0 to end
  for(w1=1;w1<=n;w1++)
  {
    slistPrepend(list,bufp);
    list->first->data[0]=0;//0 to end
  };
  free(bufp);
  return list;
}

char* slistGet(struct StringList* l,int i)
//gets the i element data from list 1-based
{
  struct StringListNode* item;
  int j;
  j=1;
  item=l->first;
  ASSERT(item!=NULL);
  do {
    if(j==i) return(item->data);
    item=item->next;
    j++;
  } while(item!=NULL);
  ASSERT(-1);//to provoke an assert if there is no item
  return NULL;
}


void slistSet(struct StringList* l,int i,const char* text)
//sets the i element of l, with text
{
  struct StringListNode* item;
  int j,len;
  j=1;
  item=l->first;
  ASSERT(item!=NULL);
  do {
    if(j==i)
    {
      len=strlen(text)+1;
      if(len>(item->maxCount))//needs realloc
      {
	free(item->data);
	item->data=(char *)malloc(sizeof(char)*len);
	ASSERT(item->data);
	item->maxCount=len;
      };
      strcpy(item->data,text);
      return;
    };
    item=item->next;
    j++;
  } while(item!=NULL);
  ASSERT(-1);// to provoke an assert if item not exist
}


void slistFree(struct StringList* l)
{
  struct StringListNode* item;
  struct StringListNode* tmp;
  item=l->first;
  while(item!=NULL)
  {
    free(item->data);
    tmp=item;
    item=item->next;
    free(tmp);
  };
  free(l);
}

void slistInsertAt(struct StringList* l,int i,const char* text)
{
  struct StringListNode* item;
  struct StringListNode* prev;
  struct StringListNode* newi;
  int j;
  item=l->first;
  prev=NULL;
  j=1;
  do {
    if(i==j)//insert here
    {
      if(prev==NULL)
      {
	slistPrepend(l,text);
      }else
      {
	newi=slistCreateNode(text);
	prev->next=newi;
	newi->next=item;
	l->count++;
      };
      return;
    };
    prev=item;
    item=item->next;
    j++;
  } while(item!=NULL);
  ASSERT(-1); //to provoke assert if item position is not found
  return;
}

void slistAppend(struct StringList* l,const char* text)
{
  struct StringListNode* item;
  struct StringListNode* newi;
  item=l->first;
  if(item!=NULL)
  {
   while(item->next!=NULL)
   {
     item=item->next;
   };
   newi=slistCreateNode(text);
   item->next=newi;
   l->count++;
  }else //list is empty
  {
    newi=slistCreateNode(text);
    l->first=newi;
    l->count++; 
  };
}

void slistRemoveAt(struct StringList* l,int i)
{
  struct StringListNode* item;
  struct StringListNode* prev;
  struct StringListNode* newi;
  int j;
  item=l->first;
  prev=NULL;
  j=1;
  do {
    if(i==j)//remove here
    {
      if(prev==NULL)
      {
	l->first=item->next;
	free(item->data);
	free(item);
      }else
      {
	prev->next=item->next;
	free(item->data);
	free(item);
      };
      l->count--;
      return;
    };
    prev=item;
    item=item->next;
    j++;
  } while(item!=NULL);
  return;
}


char* slistToArray(struct StringList* l)//ok
//writes list l to a char* buffer.
//format: int=numOfListItems, foreach item {int=num of bytes,bytes}
{
    int w1,n,i,w2;
    struct StringListNode* item;
    char* buf;
    char* b;
    n=sizeof(int);
    item=l->first;
    while(item!=NULL)
    {
      n=n+strlen(item->data)+1+sizeof(int);
      item=item->next;
    };
    l->arraySize=n;
    buf=(char *)malloc(n);
    ASSERT(buf!=NULL);
    i=0; // now write array
    memcpy(buf+i,&l->count,sizeof(int));//list size
    i=i+sizeof(int);
    item=l->first;
    while(item!=NULL)
    {
      b=item->data;
      w2=strlen(item->data)+1;
      memcpy(buf+i,&w2,sizeof(int));
      i=i+sizeof(int);
      memcpy(buf+i,b,w2);
      i=i+w2;
      item=item->next;
    };
    return buf;
}


/*DOC: struct Matrix usage:
struct Matrix* mat1=NULL;
mat1=matrixInit(mat1,3,4);//3x4 matrix
matrixSet(mat1,1,1,1.0);
matrixFree(mat1);mat1=NULL;
*/

struct Matrix* matrixInit(struct Matrix* mat,int rows,int cols)
{
	int size=rows*cols;
	int w1;
	ASSERT(rows>0);
	ASSERT(cols>0);
	mat=malloc(sizeof(struct Matrix));
	ASSERT(mat!=NULL);
	mat->rows=rows;
	mat->cols=cols;
	mat->data=malloc(sizeof(double)*size);
	ASSERT((mat->data)!=NULL);
	for(w1=0;w1<size;w1++) mat->data[w1]=0.0;
	return mat;
}

double matrixGet(struct Matrix* mat,int r,int c)
//returns row r column c of matrix mat
{
	ASSERT((r>0) && (r<=(mat->rows)));
	ASSERT((c>0) && (c<=(mat->cols)));
	return(mat->data[(r-1)*mat->cols+c-1]);
}

void matrixSet(struct Matrix* mat,int r,int c,double v)
//sets row r column c of matrix mat with value v
{
	ASSERT((r>0) && (r<=(mat->rows)));
	ASSERT((c>0) && (c<=(mat->cols)));
	mat->data[(r-1)*mat->cols+c-1]=v;
}

void matrixFree(struct Matrix* mat)
//frees allocated memory of matrix mat
{
	free(mat->data);
	free(mat);
}

void matrixZero(struct Matrix* mat)
//zeroes all elements of matrix mat
{
	int w1,n;
	n=mat->rows * mat->cols;
	for(w1=0;w1<n;w1++)
	{
		mat->data[w1]=0.0;
	};
}

struct Matrix* matrixAppendRow(struct Matrix* mat)
//returns a copy of matrix mat with a row appended
{
	struct Matrix* newmat;
	int w1,w2;
	newmat=matrixInit(newmat,mat->rows+1,mat->cols);
	for(w1=1;w1<=mat->rows;w1++)
	{
		for(w2=1;w2<=mat->cols;w2++)
		{
			matrixSet(newmat,w1,w2, matrixGet(mat,w1,w2));
		};
	};
	for(w2=1;w2<=mat->cols;w2++)
	{
		matrixSet(newmat,mat->rows+1,w2,0.0);
	};
	return(newmat);
}

struct Matrix* matrixRemoveRow(struct Matrix* mat, int r)
//returns a copy of matrix mat with row r removed
{
	struct Matrix* newmat;
	int w1,w2,w3,w4;
	if(mat->rows==1) return(NULL);
	newmat=matrixInit(newmat,mat->rows-1,mat->cols);
	w3=0;w4=0;
	for(w1=1;w1<=mat->rows;w1++)
	{
		if(w1!=r)
		{
			w3++;w4++;
			for(w2=1;w2<=mat->cols;w2++)
			{
				matrixSet(newmat,w3,w2, matrixGet(mat,w4,w2));
			};
		}else w4++;
	};
	return(newmat);
}

struct Matrix* matrixRemoveColumn(struct Matrix* mat,int c)
//returns a copy of matrix mat with col c removed
{
	struct Matrix* newmat;
	int w1,w2,w3,w4;
	if(mat->cols==1) return (NULL);
	newmat=matrixInit(newmat,mat->rows,mat->cols-1);
	w3=0;w4=0;
	for(w1=1;w1<=mat->cols;w1++)
	{
		if(w1!=c)
		{
			w3++;w4++;
			for(w2=1;w2<=mat->rows;w2++)
			{
				matrixSet(newmat,w2,w3,matrixGet(mat,w2,w4));
			};
		}else w4++;
	};
	return(newmat);
}

void matrixAddMatrix(struct Matrix* mat1,struct Matrix* mat2)
//adds mat2 to mat1. r1=r2 and c1=c2
{
	int w1,w2;
	for(w1=1;w1<=mat1->rows;w1++)
	{
		for(w2=1;w2<=mat1->cols;w2++)
		{
			matrixSet(mat1,w1,w2,matrixGet(mat1,w1,w2)+matrixGet(mat2,w1,w2));
		};
	};
}

void matrixAddNumber(struct Matrix* mat1,double v)
// adds v to all matrix mat1 elements
{
	int w1,w2;
	for(w1=1;w1<=mat1->rows;w1++)
	{
		for(w2=1;w2<=mat1->cols;w2++)
		{
			matrixSet(mat1,w1,w2,matrixGet(mat1,w1,w2)+v);
		};
	};
}

struct Matrix* matrixMulMatrix(struct Matrix* mat1,struct Matrix* mat2)
//returns the mat1 x mat2 in a new matrix. c1=r2
{
	int w1,w2,w3;
	double d1;
	struct Matrix* mat3;
	if(mat1->cols!=mat2->rows) return NULL;
	mat3=matrixInit(mat3,mat1->rows,mat2->cols);
	for(w1=1;w1<=mat1->rows;w1++) // running through the rows of the new matrix
	{
		for(w2=1;w2<=mat2->cols;w2++) //running through the cols of the new matrix
		{
			d1=0.0;
			for(w3=1;w3<=mat1->cols;w3++)
			{
				d1=d1+matrixGet(mat1,w1,w3)*matrixGet(mat2,w3,w2);
			};
			matrixSet(mat3,w1,w2,d1);
		};
	};
	return mat3;
}
struct Matrix* matrixCopy(struct Matrix* mat1)
//returns a copy of matrix mat1
{
	int w1;
	struct Matrix* newmat;
	if(mat1==NULL) return NULL;
	newmat=matrixInit(newmat,mat1->rows,mat1->cols);
	for(w1=0;w1<mat1->rows*mat1->cols;w1++)
	{
		newmat->data[w1]=mat1->data[w1];
	};
	return newmat;
}

struct Matrix* matrixTranspose(struct Matrix* mat1)
//returns the transpose of matrix mat1
{
	int w1,w2;
	struct Matrix* newmat;
	newmat=matrixInit(newmat,mat1->cols,mat1->rows);
	for(w1=1;w1<=mat1->rows;w1++)
	{
		for(w2=1;w2<=mat1->cols;w2++)
		{
			matrixSet(newmat,w2,w1, matrixGet(mat1,w1,w2));
		};
	};
	return newmat;
}

void matrixMulNumber(struct Matrix* mat1,double v)
//multiply all elements of matrix mat1 with value v
{
	int w1,w2;
	for(w1=1;w1<=mat1->rows;w1++)
	{
		for(w2=1;w2<=mat1->cols;w2++)
		{
			matrixSet(mat1,w1,w2,matrixGet(mat1,w1,w2)*v);
		};
	};
}

void matrixSwapRow(struct Matrix* mat1,int r1,int r2)
//swaps rows r1 and r2 in matrix mat1
{
	int w1;
	double d1;
	for(w1=1;w1<=mat1->cols;w1++)
	{
		d1=matrixGet(mat1,r1,w1);
		matrixSet(mat1,r1,w1,matrixGet(mat1,r2,w1));
		matrixSet(mat1,r2,w1,d1);
	};
}

void matrixAddMultiplyRow(struct Matrix* mat1,int r1,int r2,double v)
//multiplies each element of row r2 with v and adds it to r1. r1=r1+r2*v
{
	int w1;
	double d1;
	for(w1=1;w1<=mat1->cols;w1++)
	{
		d1=matrixGet(mat1,r1,w1)+matrixGet(mat1,r2,w1)*v;
		matrixSet(mat1,r1,w1,d1);
	};
}

void matrixDisplay(struct Matrix* a)
//prints out a matrix for debugging purposes. Waits a key to be pressed.
{
	char buf1[32];
	int w1,w2;
	unsigned int key;
	unsigned char* Scr1;
	Scr1=GetScreen();
	Bdisp_AllClr_VRAM();
	for(w1=1;w1<=a->rows;w1++)
	{
		for(w2=1;w2<=a->cols;w2++)
		{
			locate(1+(w2-1)*5,w1);
			sprintf(buf1,"%.2f",matrixGet(a,w1,w2));
			Print((unsigned char *)buf1);
		};
	};
	GetKey(&key);
	SetScreen(Scr1);
	free(Scr1);

}

struct Matrix* matrixConjGradient(struct Matrix* a,struct Matrix* b,double residual)
//returns X and solves AX=B with Conjugate gradient method.
{
	int w1,w2,w3,w4;
	double d1,d2;
	int n,m;
	double ak,bk,res;
	struct Matrix* matRK;
	struct Matrix* matRK1;
	struct Matrix* matXK;
	struct Matrix* matPK;
	struct Matrix* mat0;
	struct Matrix* mat1;
	struct Matrix* mat2;
	if(a->rows!=a->cols) return NULL;
	if(a->rows!=b->rows) return NULL;
	n=a->rows;
	m=b->cols;
	if(m!=1) return NULL;
	matRK=matrixInit(matRK,n,1);
	matRK1=matrixInit(matRK1,n,1);
	matXK=matrixInit(matXK,n,1);
	matPK=matrixInit(matPK,n,1);
	//set XK=X0 = {1,1,...}
	for(w1=1;w1<=n;w1++)
	{
		matrixSet(matXK,w1,1,1.0);
	};
	//set RK=R0=B-A*X0
	mat0=matrixMulMatrix(a,matXK);
	matrixMulNumber(mat0,-1.0);
	matrixAddMatrix(mat0,b);
	//set PK=P0=R0
	for(w1=1;w1<=n;w1++)
	{
		matrixSet(matPK,w1,1,matrixGet(mat0,w1,1));
		matrixSet(matRK,w1,1,matrixGet(mat0,w1,1));
	};
	matrixFree(mat0);
	//start loop
	res=1.0E+10+residual;
	while(res>residual)
	{
		//ak=rk^T rk /(pk^T A pk)
		mat0=matrixTranspose(matRK);
		mat1=matrixMulMatrix(mat0,matRK);//1x1 matrix
		d1=matrixGet(mat1,1,1);
		matrixFree(mat0);matrixFree(mat1);
		mat0=matrixTranspose(matPK);
		mat1=matrixMulMatrix(mat0,a);
		mat2=matrixMulMatrix(mat1,matPK);//1x1 matrix
		d2=matrixGet(mat2,1,1);
		matrixFree(mat0);matrixFree(mat1);matrixFree(mat2);
		ak=d1/d2;
		//xk+1=xk+ak pk
		for(w1=1;w1<=n;w1++)
		{
			matrixSet(matXK,w1,1,matrixGet(matXK,w1,1)+ak*matrixGet(matPK,w1,1));
		};
		//rk+1=rk-ak A pk
		mat0=matrixMulMatrix(a,matPK);
		for(w1=1;w1<=n;w1++)
		{
			matrixSet(matRK1,w1,1,matrixGet(matRK,w1,1)-ak*matrixGet(mat0,w1,1));
		};
		matrixFree(mat0);
		//compute residual res
		res=0.0;
		for(w1=1;w1<=n;w1++)
		{
			res=res+fabs(matrixGet(matRK1,w1,1));
		};
		//bk=rk+1^T rk+1 / (rk^T rk)
		mat0=matrixTranspose(matRK1);
		mat1=matrixMulMatrix(mat0,matRK1);//1x1 matrix
		d1=matrixGet(mat1,1,1);
		matrixFree(mat0);matrixFree(mat1);
		mat0=matrixTranspose(matRK);
		mat1=matrixMulMatrix(mat0,matRK);//1x1 matrix
		d2=matrixGet(mat1,1,1);
		matrixFree(mat0);matrixFree(mat1);
		bk=d1/d2;
		//pk+1=rk+1 + bk pk
		for(w1=1;w1<=n;w1++)
		{
			matrixSet(matPK,w1,1,matrixGet(matRK1,w1,1)+bk*matrixGet(matPK,w1,1));
		};
		//matRK=matRK1 (k=k+1)
		for(w1=1;w1<=n;w1++)
		{
			matrixSet(matRK,w1,1,matrixGet(matRK1,w1,1));
		};
	};
	//copy XK solution to B
	for(w1=1;w1<=n;w1++)
	{
		matrixSet(b,w1,1,matrixGet(matXK,w1,1));
	};
	matrixFree(matPK);
	matrixFree(matXK);
	matrixFree(matRK);

	return b;
}

struct Matrix* matrixLinearSolve(struct Matrix* a,struct Matrix* b)
//returns X from AX=B with Gauss Elimination
{
	int w1,w2,w3,w4,w5;
	int n,m;
	double piv,cl;

	if(a->rows!=a->cols) return NULL;//non square - error
	if(a->rows!=b->rows) return NULL;//a not same rows with b
	n=a->rows;
	m=b->cols;
	for(w1=1;w1<=(n-1);w1++) // rows 1 to n-1
	{
		piv=matrixGet(a,w1,w1);
		if(piv==0.0) return NULL; // zero pivot
		w2=w1+1;
		for(w3=w2;w3<=n;w3++)
		{
			cl=matrixGet(a,w3,w1)/piv;
			if(cl!=0.0)
			{
				for(w4=1;w4<=m;w4++)
				{
					matrixSet(b,w3,w4, matrixGet(b,w3,w4)-cl*matrixGet(b,w1,w4));
				};
				for(w4=w2;w4<=n;w4++)
				{
					matrixSet(a,w3,w4, matrixGet(a,w3,w4)-cl*matrixGet(a,w1,w4));
				};
			};
		};
	};
	//now backsubstitution
	for(w4=1;w4<=m;w4++)
	{
		if(matrixGet(a,n,n)==0.0) return NULL; //zero div
		matrixSet(b,n,w4, matrixGet(b,n,w4)/matrixGet(a,n,n));
	};
	for(w5=1;w5<=m;w5++)
	{
		for(w2=n-1;w2>=1;w2--)
		{
			cl=0.0;
			for(w4=w2+1;w4<=n;w4++)
			{
				cl=cl+matrixGet(a,w2,w4)*matrixGet(b,w4,w5);
			};
			if(matrixGet(a,w2,w2)==0.0) return NULL; //zero div
			matrixSet(b,w2,w5, (matrixGet(b,w2,w5)-cl)/matrixGet(a,w2,w2));
		};
	};
	return(b);
}


double matrixDeterminant(struct Matrix* a)
//returns the determinant of matrix a. Warning, function high recursive for large matrices.
{
	struct Matrix* m1;
	struct Matrix* m2;
	int n,w1;
	double det=0;
	signed int sign1;
	if(a->rows!=a->cols) return 0.0;
	n=a->rows;
	if(n==1) {det=matrixGet(a,1,1);return det;};
	if(n==2)
	{
		det=matrixGet(a,1,1)*matrixGet(a,2,2)-matrixGet(a,1,2)*matrixGet(a,2,1);
		return det;
	};
	if(n==3) 
	{
		det=matrixGet(a,1,1)*matrixGet(a,2,2)*matrixGet(a,3,3)
			+matrixGet(a,1,2)*matrixGet(a,2,3)*matrixGet(a,3,1)
			+matrixGet(a,1,3)*matrixGet(a,2,1)*matrixGet(a,3,2)
			-matrixGet(a,1,3)*matrixGet(a,2,2)*matrixGet(a,3,1)
			-matrixGet(a,1,2)*matrixGet(a,2,1)*matrixGet(a,3,3)
			-matrixGet(a,1,1)*matrixGet(a,2,3)*matrixGet(a,3,2);
		return det;
	};
	sign1=+1;
	for(w1=1;w1<=n;w1++)
	{
		m1=matrixRemoveRow(a,w1);
		m2=matrixRemoveColumn(m1,1);
		matrixFree(m1);
		det=det+sign1*matrixGet(a,w1,1)*matrixDeterminant(m2);
		matrixFree(m2);
		if(sign1==+1) sign1=-1; else sign1==+1;
	};
	return det;
}


void strMid(const char* src,int i,int n,char* dest) //ok
//returns a piece of string src to dest starting on i char and with n chars long. zero based
//example: strMid("123456",0,3,buf) returns "123"
{
	int len;
	ASSERT(i>=0);
	if(i>=strlen(src)) {dest[0]=0;return;};
	if(i+n>strlen(src)) len=strlen(src)-i; else len=n; 
	strncpy(dest,src+i,len);
	dest[len]=0;
}

int strIdx(const char* src,const char c,int instance)//ok
//returns the instance of char c in src. zero based
//example: strIdx("123.345.567",'.',1) returns 7
{
	int len=strlen(src);
	int i;
	int o=-1;
	ASSERT(instance>=0);
	for(i=0;i<len;i++)
	{
		if(src[i]==c) o++;
		if(o==instance) return i;
	};
	return -1;
}

void strList(const char* src,const char d,int i,char* dest) //ok
//returns the i item delimited with char d in src to dest. zero based.if d not exist returns src.
//example: strList("12,23,45",',',1,buf) returns "23" in buf
//example: strList("12,23,45",',',2,buf) returns "45" a=5 b=7 in buf
{
	int a,b;
	ASSERT(i>=0);
	if(i==0)
	{
		a=0;
		b=strIdx(src,d,i);
		if(b==-1) b=strlen(src);
	} else
	{
		a=strIdx(src,d,i-1);
		b=strIdx(src,d,i);
		if(a==-1) {a=0;b=0;goto end1;};
		if (b==-1) b=strlen(src);
		a++;
	};
	end1:
	dest[0]=0;
	strMid(src,a,b-a,dest);
}

int strListLength(const char* src,const char d,int i)//ok
//returns the length of the i item of src with delimiter d
//example: strListLength("12,234,4567",',',2) returns 4
{
	int a,b;
	ASSERT(i>=0);
	if(i==0)
	{
		a=0;
		b=strIdx(src,d,i);
		if(b==-1) b=a;
	} else
	{
		a=strIdx(src,d,i-1);
		b=strIdx(src,d,i);
		if(a==-1) {a=0;b=0;goto end1;};
		if (b==-1) b=strlen(src);
		a++;
	};
	end1:
	return(b-a);
}

void strListModify(const char* src,const char d,int i,const char* newitem,char* dest) //ok
//modifies the i item delimited with char d in src to dest.zero based
//example: strListModify("12,23,34",',',1,"AA",buf) returns "12,AA,34" in buf
{
	int w1,n;
	char buf[128];
	dest[0]=0;
	n=strCount(src,d);
	for(w1=0;w1<n;w1++)
	{
		if(w1==i)
		{
			strcat(dest,newitem);
		}else
		{
			strList(src,d,w1,buf);
			strcat(dest,buf);
		};
		buf[0]=d;buf[1]=0;
		if(w1!=n-1) strcat(dest,buf);
	};
}

int strCount(const char* src,const char d) //ok 
//counts the items in string src delimited with d
//example: strCount("12,23,34",',') returns 3
{
	int len=strlen(src);
	int w1,i;
	i=0;
	for(w1=0;w1<len;w1++)
	{
		if(src[w1]==d) i++;
	};
	if(i>=0) return (i+1); else return 0;
}

void strFormatValue(char* dest,const char* format, const char* src)//ok
//formats src with format and puts the result to dest
//example: strFormatValue(buf,"%f.3","1.2343678") returns "1.234"
{
	double d1;
	int w1;
	if(strIdx(format,'s',0)!=-1)//str
	{
		sprintf(dest,format,src);
	}else if(strIdx(format,'d',0)!=-1)//int
	{
		w1=atoi(src);
		sprintf(dest,format,w1);
	}else if(strIdx(format,'f',0)!=-1) //double
	{
		d1=atof(src);
		sprintf(dest,format,d1);
	}else //TODO for other types
	{
	};
}

int execMenu(const char* items) //ok
//executes a bottom line menu and returns selection
//example: execMenu("op1,op2,op3,,op5,op6"); pressing F1 will return 1
{
  char tok[16];
  unsigned char* Scr1;
  unsigned int key;
  int i,x;
  int n=strCount(items,',');
  Scr1=GetScreen();
  for(i=1;i<=n;i++)
  {
	strList(items,',',i-1,tok);
	x=(i-1)*21;
	PrintMini(x+2,55+2,(unsigned char *)tok,MINI_OVER);
	Bdisp_AreaReverseVRAM(x+1,55+1,x+21-1,63-1);
  };
  i=-1;
  while(1){
  GetKey(&key);
  if(key==KEY_CTRL_F1)
  {
	strList(items,',',0,tok);
	if(n>=1 && strlen(tok)>0) i=1;break;
  }else
  if(key==KEY_CTRL_F2)
  {
	strList(items,',',1,tok);
	if(n>=2 && strlen(tok)>0) i=2;break;
  }else
  if(key==KEY_CTRL_F3)
  {
	strList(items,',',2,tok);
	if(n>=3 && strlen(tok)>0) i=3;break;
  }else
  if(key==KEY_CTRL_F4)
  {
	strList(items,',',3,tok);
	if(n>=4 && strlen(tok)>0) i=4;break;
  }else
  if(key==KEY_CTRL_F5)
  {
	strList(items,',',4,tok);
	if(n>=5 && strlen(tok)>0) i=5;break;
  }else
  if(key==KEY_CTRL_F6)
  {
	strList(items,',',5,tok);
	if(n>=6 && strlen(tok)>0) i=6;break;
  };
  };
  SetScreen(Scr1);
  free(Scr1);
  return i;
}


void drawCircle(int x0,int y0,int r,int color)
//draws a circle on x0,y0 with r radius and color
{
  int f = 1 -r;
  int ddF_x = 1;
  int ddF_y = -2 * r;
  int x = 0;
  int y = r;
  Bdisp_SetPoint_VRAM(x0,y0+r,color);
  Bdisp_SetPoint_VRAM(x0,y0-r,color);
  Bdisp_SetPoint_VRAM(x0+r,y0,color);
  Bdisp_SetPoint_VRAM(x0-r,y0,color);
  while(x < y)
  {
    if(f >= 0)
    {
      y--;
      ddF_y += 2;
      f += ddF_y;
    }
    x++;
    ddF_x += 2;
    f += ddF_x;
    Bdisp_SetPoint_VRAM(x0 + x, y0 + y,color);
    Bdisp_SetPoint_VRAM(x0 - x, y0 + y,color);
    Bdisp_SetPoint_VRAM(x0 + x, y0 - y,color);
    Bdisp_SetPoint_VRAM(x0 - x, y0 - y,color);
    Bdisp_SetPoint_VRAM(x0 + y, y0 + x,color);
    Bdisp_SetPoint_VRAM(x0 - y, y0 + x,color);
    Bdisp_SetPoint_VRAM(x0 + y, y0 - x,color);
    Bdisp_SetPoint_VRAM(x0 - y, y0 - x,color);
  }
}


void drawLine(int x1,int y1,int x2,int y2,int color)
//draws a line from x1,y1 to x2,y2 and color
{
	if(color==1)
		Bdisp_DrawLineVRAM(x1,y1,x2,y2);
		else if(color==0) Bdisp_ClearLineVRAM(x1,y1,x2,y2);
}

void drawRect(int x1,int y1,int x2,int y2,int color)
//draws a rectangle from x1,y1 to x2,y2 and color
{
	if(color==1)
	{
	Bdisp_DrawLineVRAM(x1,y1,x2,y1);
	Bdisp_DrawLineVRAM(x2,y1,x2,y2);
	Bdisp_DrawLineVRAM(x2,y2,x1,y2);
	Bdisp_DrawLineVRAM(x1,y2,x1,y1);
	}else if(color==0)
	{
	Bdisp_ClearLineVRAM(x1,y1,x2,y1);
	Bdisp_ClearLineVRAM(x2,y1,x2,y2);
	Bdisp_ClearLineVRAM(x2,y2,x1,y2);
	Bdisp_ClearLineVRAM(x1,y2,x1,y1);
	};
}

void viewScale(double s,struct Viewport* v)
//scales rectangle of viewport v by value s
{
	double dx,dy;
	double d1,d2,d3,d4;
	dx=v->rightX - v->leftX;
	dy=v->topY - v->bottomY;
	d1=(v->leftX+v->rightX)/2.0 - dx*s/2.0;
	d2=(v->leftX+v->rightX)/2.0 + dx*s/2.0;
	d3=(v->bottomY+v->topY)/2.0 - dy*s/2.0;
	d4=(v->bottomY+v->topY)/2.0 + dy*s/2.0;
	v->leftX=d1;v->rightX=d2;v->bottomY=d3;v->topY=d4;
}


void viewLine(double x1,double y1, double x2, double y2,struct Viewport* v)
//draws a line from x1,y1 to x2,y2 in viewport v coordinates
{
	int i1,j1,i2,j2;
	i1=(x1 - v->leftX)/(v->rightX - v->leftX)*128;
	j1=(1-(y1 - v->bottomY)/(v->topY - v->bottomY))*64;
	i2=(x2 - v->leftX)/(v->rightX - v->leftX)*128;
	j2=(1-(y2 - v->bottomY)/(v->topY - v->bottomY))*64;
	drawLine(i1,j1,i2,j2,1);
}

void viewRect(double x1,double y1, double x2, double y2,struct Viewport* v)
//draws a rect from x1,y1 to x2,y2 in viewport v coordinates
{
	int i1,j1,i2,j2;
	i1=(x1 - v->leftX)/(v->rightX - v->leftX)*128;
	j1=(1-(y1 - v->bottomY)/(v->topY - v->bottomY))*64;
	i2=(x2 - v->leftX)/(v->rightX - v->leftX)*128;
	j2=(1-(y2 - v->bottomY)/(v->topY - v->bottomY))*64;
	drawRect(i1,j1,i2,j2,1);
}

double lineAngle(double x1,double y1,double x2,double y2)
//returns the angle in radians of line x1,y1 x2,y2
{
	double dx,dy;
	double d1=0.0;
	dx=x2-x1;dy=y2-y1;
	if(dx==0.0 && dy<0.0) d1=3.0*PI/2.0;
		else if(dx==0.0 && dy>0.0) d1=PI/2.0;
		else if(dx>0.0 && dy>0.0) d1=atan(dy/dx);
		else if(dx>0.0 && dy<0.0) d1=2.0*PI+atan(dy/dx);
		else if(dx<0.0) d1=PI+atan(dy/dx);
	return d1;
}

void viewPerpendicularLine(double x1,double y1,double x2,double y2, double x,double y, struct Viewport* v)
//draws a line from x,y perpendicular to line x1,y1 x2,y2 in viewport coordinates
{
	double f,t;
	double d1,d2;
	double d3,d4;
	t=lineAngle(x1,y1,x2,y2);
	d1=x1+x*cos(t);
	d2=y1+x*sin(t);
	d3=-y*sin(t);
	d4=+y*cos(t);
	viewLine(d1,d2,d1+d3,d2+d4,v);
}

void viewPrintMini(double x,double y,const char* str,int type,struct Viewport* v)
//PrintMini in viewport v coordinates
{
	int i,j;
	i=(x - v->leftX)/(v->rightX - v->leftX)*128;
	j=(1-(y - v->bottomY)/(v->topY - v->bottomY))*64;
	PrintMini(i,j,(unsigned char*)str,type);
}

int isNumberKey(unsigned int key)
//returns >0 if key is a number
{
	if(key>=48 && key<=57) return key;
	if(key==KEY_CHAR_EXP) return 69;//char 'E'
	if(key==KEY_CHAR_DP) return 46;//char '.'
	if(key==KEY_CHAR_PMINUS || key==KEY_CHAR_MINUS) return 45;//char '-'
	return 0;
}

int isAlphaKey(unsigned int key)
//returns >0 if key is a letter
{
	if(key>=65 && key<=90) return key;
	if(key==KEY_CHAR_SPACE) return 32;//char ' '
	return 0;
}

void strInput(int x,int y,char* buf,int len,const char* preinput,int hasBox) //does not save screen
//simple fullscreen input at position x,y with size=len
//exampe: strInput(1,1,buf,10,"Edit Me",1)
{
	char bb;
	char btemp[1];
	char bclear[22];
	int i=0;
	int w1;
	unsigned int key;
	if(preinput!=NULL)
	{
		strcpy(buf,preinput);i=strlen(buf);
		locate(x,y);
		Print((unsigned char *)buf);
	};
	if(len>21) len=21;
	buf[i]=0;
	while(1)
	{
		if(hasBox==1) drawRect((x-1)*CHARXS-1,(y-1)*CHARYS-1,(x+len-1)*CHARXS+1,y*CHARYS+1,1);
		bb='\1';
		GetKey(&key);
		//sprintf(b,"%d - %X",key,key);
		if(key==KEY_CTRL_EXE) break;
		w1=isNumberKey(key);
		if(w1!=0) bb=(char)w1;
		w1=isAlphaKey(key);
		if(w1!=0) bb=(char)w1;
		if(key==KEY_CTRL_DEL && i>0)
		{
			i--;
			buf[i]='\0';
			locate(x+i,y);
			Print((unsigned char *)"  ");
		};
		if(key==KEY_CTRL_AC && i>0)
		{
			i=0;
			buf[0]='\0';
			memset(bclear,32,len);//fill with spaces
			bclear[len]='\0';
			locate(x,y);
			Print((unsigned char *)bclear);
		};
		if(bb!='\1' && i<len)
		{
			i++;
			sprintf(btemp,"%c",bb);
			strcat(buf,btemp);
			locate(x+i-1,y);
			Print((unsigned char *)btemp);
		};
		Bdisp_DrawLineVRAM((x+i-1)*CHARXS,y*CHARYS-1,(x+i-1)*CHARXS+3,y*CHARYS-1);
		Bdisp_DrawLineVRAM((x+i-1)*CHARXS,y*CHARYS-2,(x+i-1)*CHARXS+3,y*CHARYS-2);
	};
	//returns char* buf
}

void matrixEditor(struct Matrix* mat,const char* headers,const char* formats) //does not save screen
//spreadsheet-like matrix editor. headers of columns can be given, and formats also.
{
	int rows,cols,r,c,r0,c0;
	unsigned int key;
	char buf[32];
	char prebuf2[32];
	int w1,w2;
	double d1;
	unsigned char* Scr2;
	
	rows=mat->rows;
	cols=mat->cols;
	r=1;c=1;
	r0=1;c0=1;

	while(1)
	{
		Bdisp_AllClr_VRAM();
		if(headers!=NULL)
		{
		for(w1=1;w1<=3;w1++)//draw horizontal headers
		{
			strList(headers,',',c0+w1-2,buf);
			PrintMini(10+(w1-1)*38+3,1,(unsigned char *)buf,MINI_OVER);
			Bdisp_AreaReverseVRAM(10+(w1-1)*38+1,0,10+(w1)*38-1,5);
		};
		};
		for(w1=1;w1<=rows;w1++)//draw vertical headers
		{
			sprintf(buf,"%d",w1+r0-1);
			PrintMini(2,5+8*(w1-1)+2,(unsigned char *)buf,MINI_OVER);
			Bdisp_AreaReverseVRAM(1,5+8*(w1-1)+1,10,5+8*(w1)-1);
		};
		for(w1=1;w1<=7;w1++)//draw items
		{
			if(w1<=rows)
			{
			for(w2=1;w2<=3;w2++)
			{
				if(w2<=cols)
				{
				d1=matrixGet(mat,r0+w1-1,c0+w2-1);
				if(formats==NULL)
				{
					if(fabs(d1)<0.0001 && d1!=0.0) sprintf(buf,"%.2E",d1); else sprintf(buf,"%.4f",d1);
				}else
				{
					strList(formats,',',c0+w2-2,prebuf2);
					sprintf(buf,prebuf2,d1);
				};
				PrintMini(10+(w2-1)*38+2+1,5+8*(w1-1)+2,(unsigned char *)buf,MINI_OVER);
				};
			};
			};
		};
		w1=r-r0+1;
		w2=c-c0+1;
		Bdisp_AreaReverseVRAM(10+(w2-1)*38+2,5+8*(w1-1)+1,10+(w2)*38-1,5+8*(w1)-1);//draw current cell
		if(c0+3-1<cols) PrintMini(123,0,(unsigned char*)"\xE6\x9B",MINI_OVER);
		if(c0>1) PrintMini(5,0,(unsigned char*)"\xE6\x9A",MINI_OVER);
label1:
		GetKey(&key);
		w2=isNumberKey(key);
		if(key==KEY_CTRL_LEFT)
		{
			if(c0>1 && c>1)
			{
				c0--;
				c--;
			}else if(c>1)
			{
				c--;
			};
		}else if(key==KEY_CTRL_RIGHT)
		{
			if(c-c0+1<3 && c<cols)
			{
				c++;
			}else if(c-c0+1==3 && c<cols)
			{
				c0++;
				c++;
			};
		}else if(key==KEY_CTRL_UP)
		{
			if(r0>1 && r>1)
			{
				r0--;
				r--;
			}else if(r>1)
			{
				r--;
			};
		}else if(key==KEY_CTRL_DOWN)
		{
			if(r-r0+1<7 && r<rows)
			{
				r++;
			}else if(r-r0+1==7 && r<rows)
			{
				r0++;
				r++;
			};
		}else if(key==KEY_CTRL_EXE || w2!=0)
		{
			if(w2!=0)
			{
				prebuf2[0]=w2;
				prebuf2[1]='\0';
			}else
			{
				d1=matrixGet(mat,r,c);
				if(fabs(d1)<0.0001 && d1!=0.0) sprintf(prebuf2,"%.5E",d1); else sprintf(prebuf2,"%.4f",d1);
			};
			buf[0]='\0';
			w1=r-r0+1;
			w2=c-c0+1;
			Scr2=GetScreen();
			PopUpWin(1);
			strInput(4,4,buf,16,prebuf2,0);
			matrixSet(mat,r,c,atof(buf));
			SetScreen(Scr2);
			free(Scr2);
			c++;
			if(c>cols) {c=1;r++;};
			if(r>rows) {r=1;c=1;};
			if(c-c0>=3 || c0>c) c0=c-3+1;
			if(r-r0>=7 || r0>r) r0=r-7+1;
			if(c0<1) c0=1;
			if(r0<1) r0=1;
		}else if(key==KEY_CTRL_EXIT)
		{
			return;
		}else
		{
			goto label1; //do not refresh if a usable key is pressed
		};
	};
}

void messageBox(const char* text,const char* title)//uses dialog no need for screen save
{
    struct StringList* dlg=NULL;
    int w1,n;
    char buf1[64];
    char buf2[64];
    n=strCount(text,'|');//counts text lines
    if(n>4) n=4;
    dlg=slistInit(n+1,60);
    for(w1=1;w1<=n;w1++)
    {
        strList(text,'|',w1-1,buf1);//text line
        sprintf(buf2,"LABEL,10,%d,%s",w1*10,buf1);
        slistSet(dlg,w1,buf2);
    };
    slistSet(dlg,n+1,"BUTTON,40,50,45,10,OK");
    dialogExec(dlg,title);
    slistFree(dlg);
}

int questionBox(const char* question,int defaultYes)//uses dialog no need for screen save
//question box dialog, returns 1 if yes is pressed
{
	unsigned int key;
	int w1=0;
	struct StringList* dlg=NULL;
	char buf[64];
	dlg=slistInit(3,32);
	strcpy(buf,"LABEL,1,30,");
	strcat(buf,question);
	slistSet(dlg,1,buf);
    if(defaultYes==1)
    {
        slistSet(dlg,2,"BUTTON,15,45,40,10,Yes");
        slistSet(dlg,3,"BUTTON,75,45,40,10,No");
    }else
    {
        slistSet(dlg,3,"BUTTON,15,45,40,10,Yes");
        slistSet(dlg,2,"BUTTON,75,45,40,10,No");
    }
    w1=dialogExec(dlg,"Question:");
	slistFree(dlg);dlg=NULL;
	if(w1==3 && defaultYes==0) return 1;
	if(w1==2 && defaultYes==1) return 1;
	return 0;
}

int multipleChoiceBox(const char* items,int curItem) //does not save screen
//multiple choice box with comma separated items
//example: multipleChoiceBox("choice 1,choice 2",1)
{
	unsigned int key;
	int i0,i,n,n0;
	int w1,w2,w3;
	char buf[32];
	DISPBOX box;
	i=curItem;
	n=strCount(items,',');
	if(n>5) n0=5; else n0=n;
	i0=curItem;
	if(i0+n0>n) i0=n-n0+1;
	PopUpWin(n0);
	w3=(8-n0)/2-(!(n0%2));
	box.left=12;box.right=115;box.top=w3*8;box.bottom=(w3+n0)*8;
	while(1)
	{
		Bdisp_AreaClr_VRAM(&box);
		for(w1=1;w1<=n0;w1++)
		{
			strList(items,',',w1+i0-2,buf);
			w2=strlen(buf);
			locate(10-w2/2,w3+w1);
			Print((unsigned char *)buf);
		};
		Bdisp_AreaReverseVRAM(box.left,(w3+i-i0)*8,box.right-7,(w3+i-i0+1)*8-1);//draw selection
		if(i0!=1)//up arrow
		{
			locate(19,w3+1);
			Print((unsigned char*)"\xE6\x9C");
		};
		if(i0+n0-1<n)//down arrow
		{
			locate(19,w3+n0);
			Print((unsigned char*)"\xE6\x9D");
		};
		GetKey(&key);
		if(key==KEY_CTRL_DOWN)
		{
			i++;
			if(i>n) i=n;
			if(i-i0+1>n0) i0++;
		}else if(key==KEY_CTRL_UP)
		{
			i--;
			if(i<1) i=1;
			if(i-i0<0) i0--;
		}else if(key==KEY_CTRL_EXE)
		{
			break;
		};
	};
	return i;
}

/*DOC Dialog usage:
struct List* dlg1=NULL;
listInit(dlg1,3,32);
listSet(dlg1,1,"LABEL,10,10,NAME:");
listSet(dlg1,2,"EDITBOX,30,10,40,8,User1,%s");
listSet(dlg1,3,"MENU,add,del,copy,,,exit");
w1=dialogExec(dlg1,"My Dialog 1");
listFree(dlg1);dlg1=NULL;
*/
int dialogExec(struct StringList* l,const char* title)
{
    //LABEL,X,Y,TEXT
    //BUTTON,X,Y,W,H,TEXT
    //EDITBOX,X,Y,W,H,TEXT,TEXTFORMAT
    //CHECKBOX,X,Y,TEXT,STATUS
    //RADIO,X,Y,NUMOFITEMS,ITEMS...,CURRENTITEM
    //DROPLIST,X,Y,W,H,NUMOFITEMS,ITEMS...,CURRENTITEM
    //LIST,X,Y,W,H,NUMOFITEMS,ITEMS...,CURRENTITEM
    //MENU,op1,op2,...,op6
	unsigned int key;
	int n;
	int w1,w2,w3,w4,w5,w6,w7,w8,x,y,w,h;
	int curItem;
	char buf[1024];
	char buff[1024];
	char buf1[128];
	char buf2[128];
	char* bufp;
	double d1;
	int hasMenu;
	int MenuOption[6];
	DISPBOX box;
	unsigned char* Scr1;
	unsigned char* Scr2;
	hasMenu=0;
	for(w1=0;w1<=5;w1++) MenuOption[w1]=0;
	curItem=1;
	n=l->count;
	Scr1=GetScreen();
	box.left=0;box.right=127;box.top=0;box.bottom=63;
	while(1)
	{
		Bdisp_AreaClr_VRAM(&box);
		//draw Title
		sprintf(buf,"< %s >",title);
		PrintMini(64-strlen(buf)/2*4-1,0,(unsigned char*)buf,MINI_OVER);
		//draw each item
		for(w1=1;w1<=n;w1++)
		{
			bufp=slistGet(l,w1);
			w2=strCount(bufp,',');//count number of options
			strList(bufp,',',0,buf);//buf now contains type "LABEL,BUTTON,EDIT,RADIO,CHECKBTN,DROPLIST,LIST,MENU"
			if(buf[0]=='L' && buf[1]=='A')// label
			{
				if(w1==curItem) curItem++;
				if(curItem>n) curItem=1;
				strList(bufp,',',1,buf); x=atoi(buf);
				strList(bufp,',',2,buf); y=atoi(buf);
				strList(bufp,',',3,buf); // text
				PrintMini(x,y,(unsigned char*)buf,MINI_OVER);
			} else if(buf[0]=='B')//button
			{
				strList(bufp,',',1,buf); x=atoi(buf);
				strList(bufp,',',2,buf); y=atoi(buf);
				strList(bufp,',',3,buf); w=atoi(buf);
				strList(bufp,',',4,buf); h=atoi(buf);
				strList(bufp,',',5,buf); // text
				PrintMini(x+w/2-strlen(buf)/2*4-1,y+h/2-4+2,(unsigned char*)buf,MINI_OVER);
				drawRect(x,y,x+w,y+h,1);
				drawRect(x,y,x+w+1,y+h+1,1);
				if(curItem==w1) Bdisp_AreaReverseVRAM(x+1,y+1,x+w,y+h);
			} else if(buf[0]=='E')//editbox
			{
				strList(bufp,',',1,buf); x=atoi(buf);
				strList(bufp,',',2,buf); y=atoi(buf);
				strList(bufp,',',3,buf); w=atoi(buf);
				strList(bufp,',',4,buf); h=atoi(buf);
				strList(bufp,',',5,buf); //text
				strList(bufp,',',5,buff); //text format
				x=x-2;y=y-2;
				PrintMini(x+w/2-strlen(buf)/2*4-1,y+h/2-4+2,(unsigned char*)buf,MINI_OVER);
				drawRect(x,y,x+w,y+h,1);
				if(curItem==w1) Bdisp_AreaReverseVRAM(x+2,y+2,x+w-2,y+h-2);
			} else if(buf[0]=='C')//checkbox
			{
				strList(bufp,',',1,buf); x=atoi(buf);
				strList(bufp,',',2,buf); y=atoi(buf);
				strList(bufp,',',3,buf); //text
				strList(bufp,',',4,buff); w3=atoi(buff);//status 0 or 1
				PrintMini(x+10,y+2,(unsigned char*)buf,MINI_OVER);
				drawRect(x+1,y+1,x+7,y+7,1);
				if(w3==1) //is checked
				{
					drawLine(x+2,y+2,x+6,y+6,1); // draw cross
					drawLine(x+2,y+6,x+6,y+2,1);
				};
				if(curItem==w1) Bdisp_AreaReverseVRAM(x,y,x+strlen(buf)*6+5,y+8);
			} else if(buf[0]=='R')//radio buttons
			{
				strList(bufp,',',1,buf); x=atoi(buf);
				strList(bufp,',',2,buf); y=atoi(buf);
				strList(bufp,',',3,buf); w3=atoi(buf); //number of choices
				w5=1;
				for(w4=1;w4<=w3;w4++)
				{
					strList(bufp,',',4+w4-1,buf); //radio choice
					if(strlen(buf)>w5) w5=strlen(buf);
					PrintMini(x+10,y+2+(w4-1)*7,(unsigned char*)buf,MINI_OVER);
					drawCircle(x+4,y+4+(w4-1)*7,3,1);
				};
				strList(bufp,',',4+w4-1,buf); w6=atoi(buf);//current choice
				drawCircle(x+4,y+4+(w6-1)*7,1,1);
				Bdisp_SetPoint_VRAM(x+4,y+4+(w6-1)*7,1);
				if(curItem==w1) Bdisp_AreaReverseVRAM(x,y,x+w5*4+8,y+7*w3+1);
			}else if(buf[0]=='D')//drop down list
			{
				strList(bufp,',',1,buf); x=atoi(buf);
				strList(bufp,',',2,buf); y=atoi(buf);
				strList(bufp,',',3,buf); w=atoi(buf);
				strList(bufp,',',4,buf); h=atoi(buf);
				strList(bufp,',',5,buf); w3=atoi(buf); //number of choices
				w4=strIdx(bufp,',',5)+1;
				w5=strIdx(bufp,',',5+w3)-1;
				strMid(bufp,w4,w5-w4+1,buff); // choice list
				strList(bufp,',',5+w3+1,buf); w6=atoi(buf); // current choice
				strList(buff,',',w6-1,buf);
				x=x-2;y=y-2;
				PrintMini(x+w/2-strlen(buf)/2*4-1,y+h/2-4+2,(unsigned char*)buf,MINI_OVER);
				drawRect(x,y,x+w,y+h,1);
				PrintMini(x+w-6,y+2,(unsigned char*)"\xE6\x93",MINI_OVER);
				if(curItem==w1) Bdisp_AreaReverseVRAM(x+2,y+2,x+w-2,y+h-2);
			}else if(buf[0]=='L' && buf[1]=='I')
			{
				strList(bufp,',',1,buf); x=atoi(buf);
				strList(bufp,',',2,buf); y=atoi(buf);
				strList(bufp,',',3,buf); w=atoi(buf);
				strList(bufp,',',4,buf); h=atoi(buf);
				strList(bufp,',',5,buf); w3=atoi(buf); //number of choices
				w4=strIdx(bufp,',',5)+1;
				w5=strIdx(bufp,',',5+w3)-1;
				strMid(bufp,w4,w5-w4+1,buff); // choice list
				strList(bufp,',',5+w3+1,buf); w6=atoi(buf); // current choice
				strList(buff,',',w6-1,buf);
				x=x-2;y=y-2;
				drawRect(x,y,x+w,y+h,1);
				w4=h/8;//number of lines
				w7=w6-w4/2;if(w7<1) w7=1;//first item to draw
				w8=w7+w4-1;// last item to draw
				if(w8>w3) w8=w3;
				for(w5=w7;w5<=w8;w5++)
				{
				    strList(buff,',',w5-1,buf);
				    PrintMini(x+w/2-strlen(buf)/2*4-1,y+2+(w5-w7)*8,(unsigned char*)buf,MINI_OVER);
				};
				drawRect(x+2,y+1+(w6-w7)*8,x+w-8,y+(w6-w7+1)*8-1,1);
				if(w8<w3) PrintMini(x+w-6,y+h-6,(unsigned char*)"\xE6\x93",MINI_OVER);
				if(w5>1) PrintMini(x+w-6,y+2,(unsigned char*)"\xE6\x92",MINI_OVER);
				//LIST is the only item that gets a rectangle instead of reverse
				if(curItem==w1) drawRect(x+1,y+1,x+w-1,y+h-1,1);
            }else if(buf[0]=='M')//menu
			{
				hasMenu=1;
				if(w1==curItem) curItem++;
				if(curItem>n) curItem=1;
				w3=w2-1; //number of menu choices
				for(w4=1;w4<=w3;w4++)
				{
					strList(bufp,',',w4,buf);//menu choice
					if(strlen(buf)>0) MenuOption[w4-1]=1;
					x=(w4-1)*21;
					PrintMini(x+2,55+2,(unsigned char*)buf,MINI_OVER);
					Bdisp_AreaReverseVRAM(x+1,55+1,x+21-1,63-1);
				};
			};

		};
		GetKey(&key);
		bufp=slistGet(l,curItem);//bufp holds the current item
		strList(bufp,',',0,buf);// get type on buf
		if(key==KEY_CTRL_DOWN && buf[0]=='L' && buf[1]=='I')
		{
		    //special case of LIST with KEY_CTRL_DOWN
		    strList(bufp,',',5,buf); w3=atoi(buf); //number of choices
		    strList(bufp,',',5+w3+1,buf); w6=atoi(buf); // current choice
		    w6++;
		    if(w6>w3) w6=1;
		    sprintf(buf1,"%d",w6);
		    strListModify(bufp,',',5+w3+1,buf1,buf);
		    slistSet(l,curItem,buf);
        }else if(key==KEY_CTRL_DOWN || key==KEY_CTRL_RIGHT)
		{
		jump1:
			curItem++;
			if(curItem>n) curItem=1;
			bufp=slistGet(l,curItem);
			strList(bufp,',',0,buf);//get type
			if((buf[0]=='L' && buf[1]=='A')|| buf[0]=='M') goto jump1;
		}else if(key==KEY_CTRL_UP && buf[0]=='L' && buf[1]=='I')
		{
		    //special case of LIST with KEY_CTRL_UP
		    strList(bufp,',',5,buf); w3=atoi(buf);//number of choices
		    strList(bufp,',',5+w3+1,buf); w6=atoi(buf);//current choice
		    w6--;
		    if(w6<1) w6=w3;
		    sprintf(buf1,"%d",w6);
		    strListModify(bufp,',',5+w3+1,buf1,buf);
		    slistSet(l,curItem,buf);
		}
		else if(key==KEY_CTRL_UP || key==KEY_CTRL_LEFT)
		{
		jump2:
			curItem--;
			if(curItem<1) curItem=n;
			bufp=slistGet(l,curItem);
			strList(bufp,',',0,buf);// get type
			if((buf[0]=='L' && buf[1]=='A')|| buf[0]=='M') goto jump2;
		}else if(key==KEY_CTRL_EXE)
		{
			//respond to EXE on each item
			bufp=slistGet(l,curItem);
			strList(bufp,',',0,buf);// get type
			if(buf[0]=='B')// if button pressed
			{
				SetScreen(Scr1);
				free(Scr1);
				return curItem;
			} else if(buf[0]=='E')//editbox
			{
			    strList(bufp,',',5,buf1); //text
				strList(bufp,',',6,buff); //text format
				Scr2=GetScreen();
				PopUpWin(1);
				buf[0]=0;strInput(4,4,buf,16,buf1,0);
				strFormatValue(buf1,buff,buf);//buf1 holds the result
				strListModify(bufp,',',5,buf1,buf);//buf now holds the updated editbox
				slistSet(l,curItem,buf); //update dialog data
				SetScreen(Scr2);
				free(Scr2);
			} else if(buf[0]=='C')//check box
			{
				strList(bufp,',',4,buff); w3=atoi(buff);//check box status
				if(w3==0) w3=1; else w3=0; //reverse status
				sprintf(buf1,"%d",w3); //buf1 hold the result
				strListModify(bufp,',',4,buf1,buf); //buf holds the updated checkbox
				slistSet(l,curItem,buf);//update dialog data
			} else if(buf[0]=='R')//radio buttons
			{
				strList(bufp,',',3,buf); w3=atoi(buf); //number of choices
				strList(bufp,',',4+w3,buf); w4=atoi(buf);//current choice
				w4++;
				if(w4>w3) w4=1;
				sprintf(buf1,"%d",w4);//buf1 holds the new radio choice
				strListModify(bufp,',',4+w3,buf1,buf); //buf holds the updated radiobox
				slistSet(l,curItem,buf);//update dialog data
			} else if(buf[0]=='D') //drop down list
			{
				strList(bufp,',',5,buf); w3=atoi(buf); //number of choices
				w4=strIdx(bufp,',',5)+1;
				w5=strIdx(bufp,',',5+w3)-1;
				strMid(bufp,w4,w5-w4+1,buff); // choice list
				strList(bufp,',',5+w3+1,buf); w6=atoi(buf); // current choice
				Scr2=GetScreen();
				w4=multipleChoiceBox(buff,w6);
				SetScreen(Scr2);
				free(Scr2);
				sprintf(buf1,"%d",w4);//buf1 holds the new choice
				strListModify(bufp,',',5+w3+1,buf1,buf);//buf holds the updated droplist
				slistSet(l,curItem,buf);//update dialog data
			} else if(buf[0]=='L' && buf[1]=='I')//list
			{
				strList(bufp,',',5,buf); w3=atoi(buf);//number of choices
				w4=strIdx(bufp,',',5)+1;
				w5=strIdx(bufp,',',5+w3)-1;
				strMid(bufp,w4,w5-w4+1,buff);//choice list
				strList(bufp,',',5+w3+1,buf); w6=atoi(buf);//current choice
				Scr2=GetScreen();
				w4=multipleChoiceBox(buff,w6);
				SetScreen(Scr2);
				free(Scr2);
				sprintf(buf1,"%d",w4); //buf1 holds the new choice
				strListModify(bufp,',',5+w3+1,buf1,buf);//buf holds the updated list
				slistSet(l,curItem,buf);//update dialog data
			};
		} else if(key==KEY_CTRL_F1)
		{
			if(hasMenu==1 && MenuOption[0]==1) {SetScreen(Scr1);free(Scr1);return 100+1;};//F1 selected
		} else if(key==KEY_CTRL_F2)
		{
			if(hasMenu==1 && MenuOption[1]==1) {SetScreen(Scr1);free(Scr1);return 100+2;};//F2 selected
		} else if(key==KEY_CTRL_F3)
		{
			if(hasMenu==1 && MenuOption[2]==1) {SetScreen(Scr1);free(Scr1);return 100+3;};//F3 selected
		} else if(key==KEY_CTRL_F4)
		{
			if(hasMenu==1 && MenuOption[3]==1) {SetScreen(Scr1);free(Scr1);return 100+4;};//F4 selected
		} else if(key==KEY_CTRL_F5)
		{
			if(hasMenu==1 && MenuOption[4]==1) {SetScreen(Scr1);free(Scr1);return 100+5;};//F5 selected
		} else if(key==KEY_CTRL_F6)
		{
			if(hasMenu==1 && MenuOption[5]==1) {SetScreen(Scr1);free(Scr1);return 100+6;};//F6 selected
		};
	};
	SetScreen(Scr1);
	free(Scr1);
	return 0;

}

void dialogSetValue(struct StringList* l,int item,const char* value)
{
	char* bufp;
	char buff[64];
	char buf1[128];
	char* buf;
	int w1;
	bufp=slistGet(l,item);
	buf=(char *)malloc(sizeof(char)*(strlen(bufp)+strlen(value)+1));
	ASSERT(buf!=NULL);
	strList(bufp,',',0,buf);//get type
	if(buf[0]=='B' || (buf[0]=='L' && buf[1]=='A') || buf[0]=='M')
	{
	      free(buf);
	      return;
	}else if(buf[0]=='E')
	{
	    //strList(bufp,',',5,buf1); //text
		strList(bufp,',',6,buff); //text format
		strFormatValue(buf1,buff,value);//buf1 holds the result
		strListModify(bufp,',',5,buf1,buf);//buf now holds the updated editbox
		slistSet(l,item,buf); //update dialog data

	}else if(buf[0]=='C')
	{
	    //strList(bufp,',',4,buf1);//checkbox status
	    strListModify(bufp,',',4,value,buf);//buf now holds the updated checkbox
	    slistSet(l,item,buf);
	}else if(buf[0]=='R')
	{
	    strList(bufp,',',3,buf1); w1=atoi(buf1); //number of choices
		//strList(bufp,',',4+w1,buf1); //current radio choice 1 based
		strListModify(bufp,',',4+w1,value,buf);
		slistSet(l,item,buf);
	}else if(buf[0]=='D')
	{
	    strList(bufp,',',5,buf1); w1=atoi(buf1); //number of choices
		//strList(bufp,',',5+w1+1,buf1); //current dropdown choice 1 based
		strListModify(bufp,',',5+w1+1,value,buf);
		slistSet(l,item,buf);
	}else if(buf[0]=='L' && buf[1]=='I')
	{
	    strList(bufp,',',5,buf1); w1=atoi(buf1); //number of choices
		//strList(bufp,',',5+w1+1,buf1); //current dropdown choice 1 based
		strListModify(bufp,',',5+w1+1,value,buf);
		slistSet(l,item,buf);
	};
	free(buf);
}

void dialogGetValue(struct StringList* l,int item,char* value)
{
	char* bufp;
	char buf[64];
	char buf1[64];
	int w1;
	bufp=slistGet(l,item);
	strList(bufp,',',0,buf);// get type
	if(buf[0]=='B' || (buf[0]=='L' && buf[1]=='A') || buf[0]=='M')
	{
		buf1[0]='\0';
	}else if(buf[0]=='E')
	{
		strList(bufp,',',5,buf1); //text
	}else if(buf[0]=='C')
	{
		strList(bufp,',',4,buf1); //checkbox status 1 or 0
	}else if(buf[0]=='R')
	{
		strList(bufp,',',3,buf1); w1=atoi(buf1); //number of choices
		strList(bufp,',',4+w1,buf1); //current radio choice 1 based
	}else if(buf[0]=='D')
	{
		strList(bufp,',',5,buf1); w1=atoi(buf1); //number of choices
		strList(bufp,',',5+w1+1,buf1); //current dropdown choice 1 based
	}else if(buf[0]=='L' && buf[1]=='I')
	{
	    strList(bufp,',',5,buf1); w1=atoi(buf1);//number of items
	    strList(bufp,',',5+w1+1,buf1);//current item 1 based
	}
	strcpy(value,buf1);
}

