/*
Beam-Frame Structural Analysis for Casio 9860SDK version 0.2
Copyright (C) 2011 Toussis Manolis (koppermind at gmail.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/


#ifndef MYLIB_H
#define MYLIB_H

#define PI 3.1415926535


extern char debugB[2048]; //debugBuf
extern unsigned int debugkey; 
extern unsigned char* debugScr;



#define ASSERT(x) if((x)==0) {Bdisp_AllClr_VRAM();sprintf(debugB,"Check:"#x);PrintMini(1,1,(unsigned char *)debugB,MINI_OVER);sprintf(debugB,"%s",__FILE__);PrintMini(1,20,(unsigned char *)debugB,MINI_OVER);sprintf(debugB,"line: %d",__LINE__);PrintMini(1,30,(unsigned char *)debugB,MINI_OVER);GetKey(&debugkey);abort();};


#define DEBUGNUM(x) if(1) {debugScr=GetScreen();Bdisp_AllClr_VRAM();sprintf(debugB,"Debug:"#x);PrintMini(1,1,(unsigned char *)debugB,MINI_OVER);sprintf(debugB,"%d , %f",x,x);PrintMini(1,10,(unsigned char *)debugB,MINI_OVER);sprintf(debugB,"%s",__FILE__);PrintMini(1,20,(unsigned char *)debugB,MINI_OVER);sprintf(debugB,"line:%d",__LINE__);PrintMini(1,30,(unsigned char *)debugB,MINI_OVER);GetKey(&debugkey);SetScreen(debugScr);free(debugScr);};

#define DEBUGSTR(x) if(1) {debugScr=GetScreen();Bdisp_AllClr_VRAM();sprintf(debugB,"Debug:"#x);PrintMini(1,1,(unsigned char *)debugB,MINI_OVER);sprintf(debugB,"%s",x);PrintMini(1,10,(unsigned char *)debugB,MINI_OVER);sprintf(debugB,"%s",__FILE__);PrintMini(1,20,(unsigned char *)debugB,MINI_OVER);sprintf(debugB,"line:%d",__LINE__);PrintMini(1,30,(unsigned char *)debugB,MINI_OVER);GetKey(&debugkey);SetScreen(debugScr);free(debugScr);};



int GetFreeMemory(void);
unsigned char* GetScreen(void);
void SetScreen(unsigned char* p);

struct StringList//Linked List of char*
{
  struct StringListNode* first;
  int count;  //number of items
  int arraySize; //used in slisttoArray
};

struct StringListNode 
{
  char* data;
  int maxCount;
  struct StringListNode* next;
};

struct StringListNode* slistCreateNode(const char* text);
void slistPrepend(struct StringList* list,const char* text);
struct StringList* slistInit(int n,int itemsize);
char* slistGet(struct StringList* l,int i);
void slistSet(struct StringList* l,int i,const char* text);
void slistFree(struct StringList* l);
void slistInsertAt(struct StringList* l,int i,const char* text);
void slistAppend(struct StringList* l,const char* text);
void slistRemoveAt(struct StringList* l,int i);
char* slistToArray(struct StringList* l);

//char* listToArray(struct List* l);

struct Matrix //1based matrix
{
	double* data;
	int rows,cols;
};
struct Matrix* matrixInit(struct Matrix* mat,int rows,int cols);
double matrixGet(struct Matrix* mat,int r,int c); //returns mat[r,c]
void matrixSet(struct Matrix* mat,int r,int c,double v); //mat[r,c]=v
void matrixFree(struct Matrix* mat);
void matrixZero(struct Matrix* mat);//fills mat with 0.0
struct Matrix* matrixAppendRow(struct Matrix* mat);
struct Matrix* matrixRemoveRow(struct Matrix* mat, int r);//remove row r
struct Matrix* matrixRemoveColumn(struct Matrix* mat,int c);//remove col c
void matrixAddMatrix(struct Matrix* mat1,struct Matrix* mat2);//adds mat2 to mat1
void matrixAddNumber(struct Matrix* mat1,double v); // mat1+v
struct Matrix* matrixMulMatrix(struct Matrix* mat1,struct Matrix* mat2);//mat1 x mat2 returns result;
struct Matrix* matrixCopy(struct Matrix* mat1);//returns a new matrix with mat1 contents
struct Matrix* matrixTranspose(struct Matrix* mat1);//returns a new matrix with transpose of mat1
void matrixMulNumber(struct Matrix* mat1,double v);// mat1*v
void matrixSwapRow(struct Matrix* mat1,int r1,int r2);//swaps rows r1 and r2
void matrixAddMultiplyRow(struct Matrix* mat1,int r1,int r2,double v);//adds row r2*v to row r1
void matrixDisplay(struct Matrix* a);//for debug purposes
struct Matrix* matrixLinearSolve(struct Matrix* a,struct Matrix* b); // A X = B returns B with solution
struct Matrix* matrixConjGradient(struct Matrix* a,struct Matrix* b,double residual); //cojugate gradient method
double matrixDeterminant(struct Matrix* a);

void strMid(const char* src,int i,int n,char* dest);//returns n chars starting with i of the src in dest
int strIdx(const char* src,const char c,int instance);//returns the index of the instance of char c
void strList(const char* src,const char d,int i,char* dest);//returns the i item zero base src with delimiter char d in dest
int strListLength(const char* src,const char d,int i);//returns the length of the i item of src with delimiter d
int strCount(const char* src,const char d);//returns the number of items in src delimited with d
void strListModify(const char* src,const char d,int i,const char* newitem,char* dest); // returns src to dest with i item changes to newitem
void strFormatValue(char* dest,const char* format, const char* src);// formats src to dest with given format respecting number formats


int execMenu(const char* items);//example execMenu("op1,op2,op3,op4,op5,next") returns selection

void drawCircle(int x0, int y0, int r, int color);
void drawLine(int x1,int y1,int x2,int y2,int color);
void drawRect(int x1,int y1,int x2,int y2, int color);

struct Viewport {
	double leftX,rightX,topY,bottomY;
};
void viewScale(double s,struct Viewport* v);
void viewLine(double x1,double y1, double x2, double y2,struct Viewport* v);//draws line
void viewRect(double x1,double y1, double x2, double y2,struct Viewport* v);//draws rect
double lineAngle(double x1,double y1,double x2,double y2); //returns angle of line
void viewPerpendicularLine(double x1,double y1,double x2,double y2, double x,double y, struct Viewport* v);//draws perpendicular line
void viewPrintMini(double x,double y,const char* str,int type,struct Viewport* v);

int isNumberKey(unsigned int key);
int isAlphaKey(unsigned int key);
void strInput(int x,int y,char* buf,int len,const char* preinput,int hasBox);//input box at x,y with max chars=len returns input in buf
void matrixEditor(struct Matrix* mat,const char* headers,const char* formats);
int questionBox(const char* question,int defaultYes);
int multipleChoiceBox(const char* items,int curItem);
void messageBox(const char* text,const char* title);


int dialogExec(struct StringList* l,const char* title);
void dialogGetValue(struct StringList* l,int item, char* value);
void dialogSetValue(struct StringList* l,int item,const char* value);


#endif
